***********************************************************************
** PAINTER Jaguar Version                                            **
** Final Source Version 08/2003                                      **
***********************************************************************
** Probably the most useful code in this game are the 2D object list **
** routines. I always have two object lists, one at $4000 and one at **
** $8000. My objects are always 32 bytes to allow them to have a     **
** forward and backward pointer which allows me to add/delete objects**
** without rebuilding the whole list. You always modify your objects **
** in the shadow objects at $8000 and these will be copied over to   **
** the active list at VBL time. You will find routines that modify   **
** X and Y coordinates of objects in the shadow object list as an    **
** example. You should always leave 1 object in the active list, the **
** dummy object.                                                     **
**                                                                   **
** When you don't rebuild the entire object list every VBL you will  **
** find that you have lots of processor time to do other things!     **
**                                                                   **
** Hopefully, this code will also serve as a decent example of how   **
** to use the Sinister sound routines.                               **
** See play.s for lots more help!                                    **
***********************************************************************

num_real_objects       .EQU     8
num_objects            .EQU     255
shadow_base            .EQU     (num_objects+1)*$20
object_base            .EQU     $4000
scr_x                  .equ     16
scr_y                  .equ     0*2

                .include "all.inc"
                .gpu
                .include "gupdate.inc"
                .68000

                .text
;               lea     pro_data,a0
;               lea     check_up,a0
;               lea     screen_32k,a0
Begin:          lea     $1ffffc,a7              ;set stack up high
                bsr     wipeout                 ;kill everything
                bsr     InitVideo               ;set up video
                move.w  #0,curr_mod             ;set mod table pointer to 0
                lea     mod_table,a0            ;start of mod table
                move.w  curr_mod,d0             ;get curr mod
                lsl.l   #2,d0                   ;*4 for longs
                add.l   d0,a0                   ;find mod in table
                move.l  (a0),a1                 ;load mod address to A1
                jsr     player                  ;init player
                bsr     wait_vbl_raw            ;wait one vbl
                bsr     wait_vbl_raw            ;and another
                move.w  #160,d0                 ;0-255 volume
                move.w  d0,music_vol            ;save music volume
                jsr     player+$14              ;set music volume
                move.w  #220,d0                 ;0-255 volume
                move.w  d0,effects_vol          ;save effects volume
                jsr     player+$18              ;set efx volume
                move.w  #(255-160)-1,d7         ;set volume sprite to 160
.loop:          jsr     sub_mus_spr             ;call this many times
                dbf     d7,.loop
                move.w  #(255-220)-1,d7         ;set effect volume sprtite to 220
.loop2:         jsr     sub_fx_spr              ;call many times
                dbf     d7,.loop2
                bsr     wait_vbl_raw            ;wait 1 vbl
                bsr     wait_vbl_raw            ;and again
setup_stuff:    bsr     build_predef_objects    ;convert objects from readable format to silly object list format
                bsr     init_object_list        ;create initial object list
                bsr     copy_olist              ;copy to active object list space
                clr.l   vbl
                move.w  #%011011000111,VMODE    ;6C7 16bit Colour
                bsr     IntInit                 ;start interrupts

                moveq   #0,d0                   ;add dummy object
                moveq   #-1,d1                  ;end of objects
                bsr     add_object              ;do it

                move.l  #-1,pro_obs
                move.w  #3,NO_OF_LIVES          ;wonder what this does?
                sf      cheat                   ;this isn't used... who needs cheats
                st      control
                bsr     game_init               ;do some game initialisation
                clr.w   START_LEVEL             ;reset level to 0

                clr.w   passwd_pos              ;start with blank password
                lea     passwd_asc,a0
                moveq   #7,d0
.clr:           clr.b   (a0)+
                dbf     d0,.clr

                lea     score_asc,a0            ;setup initial scores
                move.b  #'0',(a0)
                move.b  #'0',1(a0)
                move.b  #'0',2(a0)
                move.b  #'0',3(a0)
                move.b  #'9',4(a0)
                move.b  #'9',5(a0)
                move.b  #'9',6(a0)
                move.b  #'9',7(a0)
                lea     menu_score1,a0
                move.b  #'0',(a0)
                move.b  #'0',1(a0)
                move.b  #'0',2(a0)
                move.b  #'1',3(a0)
                move.b  #'0',4(a0)
                move.b  #'0',5(a0)
                move.b  #'0',6(a0)
                move.b  #'0',7(a0)
                lea     menu_score2,a0
                move.b  #'0',(a0)
                move.b  #'0',1(a0)
                move.b  #'0',2(a0)
                move.b  #'0',3(a0)
                move.b  #'0',4(a0)
                move.b  #'0',5(a0)
                move.b  #'0',6(a0)
                move.b  #'0',7(a0)
                lea     menu_score3,a0
                move.b  #'0',(a0)
                move.b  #'0',1(a0)
                move.b  #'0',2(a0)
                move.b  #'1',3(a0)
                move.b  #'0',4(a0)
                move.b  #'0',5(a0)
                move.b  #'0',6(a0)
                move.b  #'0',7(a0)
                lea     menu_score4,a0
                move.b  #'0',(a0)
                move.b  #'0',1(a0)
                move.b  #'0',2(a0)
                move.b  #'0',3(a0)
                move.b  #'0',4(a0)
                move.b  #'0',5(a0)
                move.b  #'0',6(a0)
                move.b  #'0',7(a0)

                move.w  #0,starttext

painter_mainloop:
                clr.w   START_LEVEL
                bsr     intro
                cmp.w   #1,d0
                beq     run_game
                cmp.w   #2,d0
                beq     reset
                bsr     wipeout
                rts
run_game:       bsr     game_init
                bsr     game
                bra     painter_mainloop

reset:          jmp     Begin

wipeout:        move.w  #$2700,sr
                move.l  #0,INT1
                move.w  #-1,VI
                move.w  #$6c1,VMODE
                lea     $150000,a0
                move.l  a0,d0
                swap    d0
                move.l  #0,(a0)+
                move.l  #4,(a0)+
                move.l  #OLP,a0
                lea     $f00004,a1
                move.l  (a1),d1
.wait_vbl:      move.l  (a1),d2
                cmp.w   d2,d1
                beq.s   .wait_vbl

                move.l  d0,(a0)
                moveq.l #0,d0
                move.l  d0,L_I2S
                move.l  d0,R_I2S
                move.l  d0,G_CTRL
                move.l  d0,D_CTRL
                move.l  d0,G_FLAGS
                move.l  d0,D_FLAGS
                move.l  d0,A1_FLAGS
                move.l  d0,A1_CLIP
                move.l  d0,A2_FLAGS
                move.l  #$70007,G_END
                move.l  #$70007,D_END
clear_gpu_ram:
                lea     $f03000,a5              ; start of RAM
                lea     $f04000,a6              ; end of RAM
                moveq.l #0,d0
                moveq.l #0,d1
                moveq.l #0,d2
                moveq.l #0,d3
                moveq.l #0,d4
                moveq.l #0,d5
                moveq.l #0,d6
                moveq.l #0,d7
.loop:          movem.l d0-d7,-(a6)
                movem.l d0-d7,-(a6)
                movem.l d0-d7,-(a6)
                movem.l d0-d7,-(a6)
                cmp.l   a5,a6
                bgt.s   .loop
clear_dsp_ram:
                lea     $f1b000,a5              ; start of RAM
                lea     $f1d000,a6              ; end of RAM
                moveq.l #0,d0
                moveq.l #0,d1
                moveq.l #0,d2
                moveq.l #0,d3
                moveq.l #0,d4
                moveq.l #0,d5
                moveq.l #0,d6
                moveq.l #0,d7
.loop:          movem.l d0-d7,-(a6)
                movem.l d0-d7,-(a6)
                movem.l d0-d7,-(a6)
                movem.l d0-d7,-(a6)
                cmp.l   a5,a6
                bgt.s   .loop
                rts

max_objlist:    lea     HC,a1
                move.l  (a1),d1
.wait_vbl:      move.l  (a1),d2
                cmp.w   d2,d1
                beq.s   .wait_vbl
                move.l  #object_base<<16,OLP
                rts

min_objlist:    rts
                move.l  (a1),d1
.wait_vbl:      move.l  (a1),d2
                cmp.w   d2,d1
                beq.s   .wait_vbl
                move.l  #obj_stop,d0
                swap    d0
                move.l  d0,OLP
                rts



wait_int:       move.w  intcount,d0
.ww:            cmp.w   intcount,d0
                beq.s   .ww
                rts

copy_olist:

                move.l  #0,d7
                move.l  #object_base+shadow_base,A2_BASE
                move.l  #object_base,A1_BASE
                move.l  d7,A1_PIXEL
                move.l  d7,A2_PIXEL
                move.w  #1,d0
                swap    d0
                move.w  #num_objects,d0
                add.w   #1,d0
                lsl.w   #3,d0
                move.l  d0,B_COUNT
                move.l  #PIXEL32|XADDPHR,d6
                move.l  d6,A1_FLAGS
                move.l  d6,A2_FLAGS
                move.l  #SRCEN|LFU_S,d7
                move.l  #0,A1_CLIP
                move.l  d7,B_CMD

                rts

IntInit:        or.w    #$700,sr
                move.w  #0,INT1
                move.l  #Frame,LEVEL0
                move.w  a_vde,d0
                ori.w   #1,d0
                move.w  d0,VI
                move.w  #1,INT1
                move.w  sr,d0
                and.w   #$f8ff,d0
                move.w  d0,sr
                rts

Frame:          movem.l d0-d7/a0-a7,-(sp)
                add.w   #1,intcount
                bsr     copy_olist
                lea     vbl,a0
                tst.l   (a0)
                beq.s   .no_vbl
                move.l  (a0),a0
                jsr     (a0)
.no_vbl:        jsr     player+8
                move.w  #$101,INT1
                move.w  #0,INT2
                movem.l (sp)+,d0-d7/a0-a7
                rte

readpad:        move.l  #$f0fffffc,d1
                moveq   #-1,d2
                move.w  #$81fe,JOYSTICK
                move.l  JOYSTICK,d0
                or.l    d1,d0
                ror.l   #4,d0
                and.l   d0,d2
                move.w  #$81fd,JOYSTICK
                move.l  JOYSTICK,d0
                or.l    d1,d0
                ror.l   #8,d0
                and.l   d0,d2
                move.w  #$81fb,JOYSTICK
                move.l  JOYSTICK,d0
                or.l    d1,d0
                rol.l   #6,d0
                rol.l   #6,d0
                and.l   d0,d2
                move.w  #$81f7,JOYSTICK
                move.l  JOYSTICK,d0
                or.l    d1,d0
                rol.l   #8,d0
                and.l   d0,d2
                moveq   #-1,d1
                eor.l   d2,d1
                move.l  joycur,d0
                move.l  joycur,d2
                and.l   #$00f00000,d2
                move.l  d1,joycur
                eor.l   d1,d0
                and.l   d1,d0
                or.l    d2,d0
                move.l  d0,joyedge
                move.l  #$0ffffff3,d1
                moveq.l #-1,d2
                move.w  #$817f,JOYSTICK
                move.l  JOYSTICK,d0
                or.l    d1,d0
                rol.b   #2,d0
                ror.l   #8,d0
                and.l   d0,d2
                move.w  #$81bf,JOYSTICK
                move.l  JOYSTICK,d0
                or.l    d1,d0
                rol.b   #2,d0
                ror.l   #8,d0
                ror.l   #4,d0
                and.l   d0,d2
                move.w  #$81df,JOYSTICK
                move.l  JOYSTICK,d0
                or.l    d1,d0
                rol.b   #2,d0
                rol.l   #8,d0
                and.l   d0,d2
                move.w  #$81ef,JOYSTICK
                move.l  JOYSTICK,d0
                or.l    d1,d0
                rol.b   #2,d0
                rol.l   #4,d0
                and.l   d0,d2
                moveq   #-1,d1
                eor.l   d2,d1
                move.l  joycur+4,d0
                move.l  joycur+4,d2
                and.l   #$00f00000,d2
                move.l  d1,joycur+4
                eor.l   d1,d0
                and.l   d1,d0
                or.l    d2,d0
                move.l  d0,joyedge+4
                move.l  joycur,d0
                moveq   #KEY_0,d1
                btst.l  d1,d0
                beq     .not0
                cmp.w   #0,zero_lock
                bne.s   .get_out
                move.w  #1,zero_lock
                eor.w   #1,zero_press
                cmp.w   #0,zero_press
                beq.s   .musoff
                jsr     player+$1c
                bra.s   .get_out
.musoff:        jsr     player+$20
                bra.s   .get_out
.not0:          move.w  #0,zero_lock
.get_out:       rts


InitVideo:      movem.l d0-d6,-(sp)
                move.w  CONFIG,d0
                andi.w  #VIDTYPE,d0
                beq.s   .palvals
                move.w  #NTSC_HMID,d2
                move.w  #NTSC_WIDTH,d0
                move.w  #NTSC_VMID,d6
;               move.w  #NTSC_HEIGHT,d4
                move.w  #200,d4
                move.w  #60,d7
                bra.s   .calc_vals
.palvals:       move.w  #PAL_HMID,d2
                move.w  #PAL_WIDTH,d0
                move.w  #PAL_VMID,d6
;               move.w  #PAL_HEIGHT,d4
                move.w  #200,d4
                move.w  #50,d7
.calc_vals:     move.w  d0,d1
                asr     #1,d1
                sub.w   d1,d2
                add.w   #4,d2
                sub.w   #1,d1
                ori.w   #$400,d1
                move.w  d1,HDE
                move.w  d2,HDB1
                move.w  d2,HDB2
                move.w  d6,d5
                sub.w   d4,d5
                move.w  d5,a_vdb
                add.w   d4,d6
                move.w  d6,a_vde
                move.w  a_vdb,VDB
                move.w  d6,VDE
                move.l  #0,BORD1
                move.w  #0,BG
                add.w   d5,LEVEL_TEXT+2
                add.w   d5,TIME_TEXT+2
                add.w   d5,LEVEL_NAME_TEXT+2
                add.w   d5,PASSWORD_TEXT+2
                add.w   d5,GAME_OVER_TEXT+2
                add.w   d5,GAME_ABORTED_TEXT+2
                add.w   d5,WOW_TEXT+2
                add.w   d5,WOW2_TEXT+2
                add.w   d5,TIME_BONUS_PRINT+2
                add.w   d5,TIMEOUT_TEXT+2
                move.w  d7,time_loop_value
                movem.l (sp)+,d0-d6
                rts

wait_vbl_raw:
                lea     $f00004,a1
                move.l  (a1),d1
.wait_vbl:      move.l  (a1),d2
                cmp.w   d2,d1
                beq.s   .wait_vbl
                rts

player:         .include "play.s"                ;music/sound effects routine

init_object_list:
                move.w  #0,curr_objects
                move.l  #0,next_object
                move.l  #object_base,a0
                move.w  #num_objects,d7
                subq.w  #1,d7
                move.l  #0,d6
.clear1:        move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                dbf     d7,.clear1
                move.l  a0,obj_list_end
                move.l  #0,(a0)+
                move.l  #4,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  #-1,(a0)+
                move.l  #-1,(a0)+
                move.w  #num_objects,d7
                subq.w  #1,d7
                move.l  #0,d6
.clear2:        move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                dbf     d7,.clear2
                move.l  a0,stop_object
                move.l  a0,d2
                move.l  #0,(a0)+
                move.l  #4,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  d6,(a0)+
                move.l  #-1,(a0)+
                move.l  #-1,(a0)+
                lea     end_ptrs,a0
                move.l  #object_base+(num_objects*32),a1
                move.w  #num_objects,d7
                subq.w  #1,d7
.fill:          sub.l   #32,a1
                move.l  a1,-(a0)
                dbf     d7,.fill
                move.l  #obj_ptr_table,next_object
                lea     object_base,a0
                lea     branch_object,a5
                clr.l   d1
                move.l  #$8003,d0
                andi.l  #$3FFFF8,d2
                move.l  d2,d3
                swap    d2
                clr.w   d2
                lsl.l   #5,d2
                or.l    d2,d0
                lsr.l   #8,d3
                lsr.l   #3,d3
                or.l    d3,d1
                move.w  a_vde,d3
                lsl.w   #3,d3
                or.w    d3,d0
                move.l  d1,(a5)+
                move.l  d0,(a5)+
                andi.l  #$FF000007,d0
                ori.l   #$4003,d0
                move.w  a_vdb,d3
                lsl.w   #3,d3
                or.w    d3,d0
                move.l  d1,(a5)+
                move.l  d0,(a5)+
                clr.l   d1
                move.l  #$0003,d0
                move.l  #$4020,d2
                andi.l  #$3FFFF8,d2
                move.l  d2,d3
                swap    d2
                clr.w   d2
                lsl.l   #5,d2
                or.l    d2,d0
                lsr.l   #8,d3
                lsr.l   #3,d3
                or.l    d3,d1
                move.w  #$7ff,d3
                lsl.w   #3,d3
                or.w    d3,d0
                move.l  d1,(a5)+
                move.l  d0,(a5)+
                move.l  #0,d0
                move.l  #-1,d1
                bsr     add_first_branches
                rts
add_first_branches:
                movem.l d0-d6/a0-a6,-(a7)
                move.l  next_object,a6
                move.l  (a6)+,a1
                move.l  a1,a4
                move.l  a6,next_object
                add.l   #shadow_base,a4
                lea     branch_object,a0
                bra     copy
add_object:     movem.l d0-d6/a0-a6,-(a7)
                move.w  #1,no_copy
                move.l  next_object,a6
                move.l  (a6)+,a1
                move.l  a1,a4
                move.l  a6,next_object
                add.l   #shadow_base,a4
                lea     pre_def_objects,a0
                lsl.w   #5,d0
                add.w   d0,a0
copy:           move.l  (a0)+,(a4)
                move.l  (a0)+,4(a4)
                move.l  (a0)+,8(a4)
                move.l  (a0)+,12(a4)
                move.l  (a0)+,16(a4)
                move.l  (a0)+,20(a4)
                move.l  (a0)+,24(a4)
                move.l  (a0)+,28(a4)
                tst.w   d1
                bpl.s   .middle
                move.l  obj_list_end,d1
.middle:        move.l  d1,a2
                move.l  d1,a5
                add.l   #shadow_base,a5
                tst.w   curr_objects
                bne.s   .yes
.no:            move.l  d1,d2
                move.l  #-1,24(a4)
                move.l  d1,28(a4)
                move.l  a1,24(a5)
                bra     .exit
.yes:           move.l  24(a5),a0
                move.l  a0,a3
                add.l   #shadow_base,a3
                move.b  3(a3),3(a4)
                move.b  4(a3),4(a4)
                move.l  a1,d2
                lsl.l   #5,d2
                and.l   #$f80000FF,2(a3)
                or.l    d2,2(a3)
                move.l  a0,24(a4)
                move.l  28(a3),28(a4)
                move.l  a1,28(a3)
                move.l  a1,24(a5)
.exit:          add.w   #1,curr_objects
                move.l  a1,d7
                movem.l (a7)+,d0-d6/a0-a6
                move.w  #0,no_copy
                rts
del_object:     movem.l d0-d6/a0-a6,-(a7)
                move.w  #1,no_copy
                move.l  next_object,a6
                move.l  d0,-(a6)
                move.l  d0,a1
                move.l  d0,a4
                add.l   #shadow_base,a4
                move.l  a6,next_object
                sub.w   #1,curr_objects
                tst.w   curr_objects
                bne.s   .yes
                move.l  #$DEADDEAD,d0
                illegal
.yes:           move.l  24(a4),a0
                move.l  a0,a3
                add.l   #shadow_base,a3
                move.l  28(a4),a2
                move.l  a2,a5
                add.l   #shadow_base,a5
                move.b  3(a4),3(a3)
                move.b  4(a4),4(a3)
                move.l  28(a4),28(a3)
                move.l  24(a4),24(a5)
                move.w  #0,no_copy
                movem.l (a7)+,d0-d6/a0-a6
                rts
build_predef_objects:
                move.l  #num_real_objects,d6
                lea     raw_obj_data,a1
                moveq   #12,d3
                moveq   #10,d4
                moveq   #11,d5
                move.l  #object_base,d7
                lsr.l   #3,d7
                move.l  #pre_def_objects,a2
                sub.w   #1,d6
.proc_obj:      tst.b   O_TYPE(a1)
                bmi     .next_obj
                move.l  O_DATA(a1),d0
                clr.l   d2
.02:            lsr.l   #3,d0
                lsl.l   d5,d0
                add.l   #4,d7
                move.l  d7,d1
                lsr.l   #8,d1
                or.l    d1,d0
                move.l  d0,(a2)+
                move.b  d7,d0
                lsl.l   d4,d0
                move.w  O_HEIGHT(a1),d1
                cmpi.b  #1,O_TYPE(a1)
                bne.b   .03
                sub.w   #1,d1
.03:            sub.w   d2,d1
                bmi.b   .04
                or.w    d1,d0
.04:            lsl.l   d5,d0
                move.w  O_YPOS(a1),d1
                bmi.b   .05
                and.w   #$7ff,d1
                or.w    d1,d0
.05:            lsl.l   #3,d0
                or.b    O_TYPE(a1),d0
                move.l  d0,(a2)+
                move.b  O_FIRSTPIX(a1),d0
                lsl.l   #4,d0
                or.b    O_FLAGS(a1),d0
                lsl.l   #7,d0
                or.b    O_INDEX(a1),d0
                lsl.l   #6,d0
                move.w  O_IWIDTH(a1),d1
                lsr.w   #4,d1
                or.b    d1,d0
                move.l  d0,(a2)+
                move.w  O_IWIDTH(a1),d0
                and.w   #$F,d0
                lsl.l   d4,d0
                or.w    O_DWIDTH(a1),d0
                lsl.l   #3,d0
                or.b    O_PITCH(a1),d0
                lsl.l   #3,d0
                or.b    O_DEPTH(a1),d0
                lsl.l   d3,d0
                move.w  O_XPOS(a1),d1
                and.w   #$fff,d1
                or.w    d1,d0
                move.l  d0,(a2)+
                move.l  O_SCALE(a1),4(a2)
                adda.l  #16,a2
.next_obj:      adda.l  #OBJSIZE,a1
                dbra    d6,.proc_obj
                rts

clear_screen:   lea     screen_32k,a0
                move.w  #$1f3f,d0
.clr:           clr.l   (a0)+
                dbf     d0,.clr
                rts

set_palette16:  lea     CLUT,a1
                moveq   #7,d0
.lp:            move.l  (a0)+,(a1)+
                dbf     d0,.lp
set_palette32:  lea     ball_pal,a0
                lea     CLUT+64,a1
                move.w  #15,d0
.lp:            move.l  (a0)+,(a1)+
                dbf     d0,.lp
                rts
fade_2_pal:     lea     CLUT,a1
.more:          bsr     wait_int
                moveq   #15,d1
                sf      d0
.lp:            move.w  (a0)+,d2
                cmp.w   (a1)+,d2
                beq.s   .next
                st      d0
                move.b  d2,d3
                and.w   #%111111,d3
                lsr.w   #6,d2
                move.b  d2,d4
                and.w   #%11111,d4
                lsr.w   #5,d2
                move.w  -2(a1),d5
                move.b  d5,d6
                and.w   #%111111,d6
                lsr.w   #6,d5
                move.b  d5,d7
                and.w   #%11111,d7
                lsr.w   #5,d5
.red:           cmp.w   d2,d5
                beq.s   .green
                bgt.s   .sub_r
                addq.w  #1,d5
                bra.s   .green
.sub_r:         subq.w  #1,d5
.green:         cmp.w   d3,d6
                beq.s   .blue
                bgt.s   .sub_g
                addq.w  #1,d6
                bra.s   .blue
.sub_g:         subq.w  #1,d6
.blue:          cmp.w   d4,d7
                beq.s   .done
                bgt.s   .sub_b
                addq.w  #1,d7
                bra.s   .done
.sub_b:         subq.w  #1,d7
.done:          lsl.w   #5,d5
                or.w    d7,d5
                lsl.w   #6,d5
                or.w    d6,d5
                move.w  d5,-2(a1)
.next:          dbf     d1,.lp
                lea     -32(a0),a0
                lea     -32(a1),a1
                move.w  (a1),BG
                tst.b   d0
                bne.s   .more
                rts


intro:
                bsr     load_gpucode
                bsr     clear_screen
                moveq   #0,d0
                moveq   #0,d1
                moveq   #0,d2
                moveq   #0,d3
                lea     mod_table,a1
                move.w  curr_mod,d0
                lsl.l   #2,d0
                add.l   d0,a1
                move.l  (a1),a0
;               lea     mod1,a0
                jsr     player+4                ;init new mod
                lea     intro_pal,a0
                bsr     set_palette16
                lea     ball_obj,a0
                move.w  #39,d6                  ; 31 dog 39 fuji
.balls:         moveq   #6,d0                   ; 39 cup
                moveq   #-1,d1
                bsr     add_object
                add.l   #shadow_base,d7
                move.l  d7,(a0)+
                dbf     d6,.balls
                moveq   #2,d0
                moveq   #-1,d1
                bsr     add_object
                add.l   #shadow_base,d7
                move.l  d7,scr_obj
                bsr     wait_int
                bsr     wait_int
                bsr     max_objlist

                bsr     lower_logo
                bsr     raise_cr
                bsr     wait_int
                move.w  #2,xrot
                move.w  #3,yrot
                move.w  #1,zrot
                move.w  #1,rotspeed
main:
                lea     score_asc,a0
                moveq   #0,d0
                moveq   #0,d1
                move.b  (a0)+,d0
                lsl.l   #8,d0
                move.b  (a0)+,d0
                lsl.l   #8,d0
                move.b  (a0)+,d0
                lsl.l   #8,d0
                move.b  (a0)+,d0
                move.b  (a0)+,d1
                lsl.l   #8,d1
                move.b  (a0)+,d1
                lsl.l   #8,d1
                move.b  (a0)+,d1
                lsl.l   #8,d1
                move.b  (a0)+,d1
                lea     menu_score1,a0
                moveq   #0,d2
                moveq   #0,d3
                move.b  (a0)+,d2
                lsl.l   #8,d2
                move.b  (a0)+,d2
                lsl.l   #8,d2
                move.b  (a0)+,d2
                lsl.l   #8,d2
                move.b  (a0)+,d2
                move.b  (a0)+,d3
                lsl.l   #8,d3
                move.b  (a0)+,d3
                lsl.l   #8,d3
                move.b  (a0)+,d3
                lsl.l   #8,d3
                move.b  (a0)+,d3
                cmp.l   d0,d2
                bgt     .set_last
                blt.s   .set_high
                cmp.l   d1,d3
                blt.s   .set_high
                bra     .set_last
.set_high:      lea     score_asc,a0
                lea     menu_score1,a1
                lea     menu_score2,a2
                lea     menu_score3,a3
                lea     menu_score4,a4
                move.b  (a0),(a1)
                move.b  (a0),(a2)
                move.b  (a0),(a3)
                move.b  (a0),(a4)
                move.b  1(a0),1(a1)
                move.b  1(a0),1(a2)
                move.b  1(a0),1(a3)
                move.b  1(a0),1(a4)
                move.b  2(a0),2(a1)
                move.b  2(a0),2(a2)
                move.b  2(a0),2(a3)
                move.b  2(a0),2(a4)
                move.b  3(a0),3(a1)
                move.b  3(a0),3(a2)
                move.b  3(a0),3(a3)
                move.b  3(a0),3(a4)
                move.b  4(a0),4(a1)
                move.b  4(a0),4(a2)
                move.b  4(a0),4(a3)
                move.b  4(a0),4(a4)
                move.b  5(a0),5(a1)
                move.b  5(a0),5(a2)
                move.b  5(a0),5(a3)
                move.b  5(a0),5(a4)
                move.b  6(a0),6(a1)
                move.b  6(a0),6(a2)
                move.b  6(a0),6(a3)
                move.b  6(a0),6(a4)
                move.b  7(a0),7(a1)
                move.b  7(a0),7(a2)
                move.b  7(a0),7(a3)
                move.b  7(a0),7(a4)
                bra.s   .st
.set_last:      lea     score_asc,a0
                lea     menu_score2,a2
                lea     menu_score4,a4
                move.b  (a0),(a2)
                move.b  (a0),(a4)
                move.b  1(a0),1(a2)
                move.b  1(a0),1(a4)
                move.b  2(a0),2(a2)
                move.b  2(a0),2(a4)
                move.b  3(a0),3(a2)
                move.b  3(a0),3(a4)
                move.b  4(a0),4(a2)
                move.b  4(a0),4(a4)
                move.b  5(a0),5(a2)
                move.b  5(a0),5(a4)
                move.b  6(a0),6(a2)
                move.b  6(a0),6(a4)
                move.b  7(a0),7(a2)
                move.b  7(a0),7(a4)
.st:            cmp.w   #0,starttext
                beq     .norm
                lea     startup_txt2,a0
                bra.s   main_ent
.norm:          lea     startup_txt,a0
main_ent:       bsr     clear_itxt_area
                bsr     print_string
                move.w  #0,keyloop
                lea     point_table,a0
                move.l  a0,ptable

intro_loop:    ;move.l  #$77777777,BG
                move.w  #2,xrot
                move.w  #3,yrot
                move.w  #1,zrot
                move.w  #1,rotspeed
                cmp.w   #20,keyloop
                blt     .ww
                bsr     readpad
                move.l  joycur,d0
                moveq   #FIRE_A,d1
                btst.l  d1,d0
                bne     password
                moveq   #FIRE_C,d1
                btst.l  d1,d0
                beq     .fireb
                bclr    d1,d0
                bra     credit_ent
.fireb:         moveq   #FIRE_B,d1
                btst.l  d1,d0
                bne     start_game
.test1:         moveq   #KEY_1,d1
                btst.l  d1,d0
                beq     .test_opt
                bclr    d1,d0
                bra     instr_ent
.test_opt:      moveq   #OPTION,d1
                btst.l  d1,d0
                beq     .ball
                moveq   #KEY_STAR,d1
                btst.l  d1,d0
                beq     .ball
                bra     control_ent
.ww:
.ball:          move.w  xrot,d7
                add.w   d7,rotx
                move.w  yrot,d7
                add.w   d7,roty
                move.w  zrot,d7
                add.w   d7,rotz
                bsr     ball_code
                bsr     wait_int
                add.w   #1,keyloop
                bra     intro_loop

credit_ent:     lea     credit_txt,a0
                bsr     clear_itxt_area
                bsr     print_string
                move.w  #0,keyloop
                lea     point_table2,a0
                move.l  a0,ptable
cred_loop:     ;move.l  #$77777777,BG
                cmp.w   #20,keyloop
                blt     .ww
                bsr     readpad
                move.l  joycur,d0
                moveq   #FIRE_A,d1
                btst.l  d1,d0
                bne     main
                moveq   #FIRE_C,d1
                btst.l  d1,d0
                bne     main
                moveq   #FIRE_B,d1
                btst.l  d1,d0
                bne     main
.ww:
.ball:          move.w  xrot,d7
                add.w   d7,rotx
                move.w  yrot,d7
                add.w   d7,roty
                move.w  zrot,d7
                add.w   d7,rotz
                bsr     ball_code
                bsr     wait_int
                add.w   #1,keyloop
                bra     cred_loop

instr_ent:      lea     instr_txt,a0
                bsr     clear_itxt_area
                bsr     print_string
                move.w  #0,keyloop
                lea     point_table3,a0
                move.l  a0,ptable
inst_loop:     ;move.l  #$77777777,BG
                cmp.w   #20,keyloop
                blt     .ww
                bsr     readpad
                move.l  joycur,d0
                moveq   #FIRE_A,d1
                btst.l  d1,d0
                bne     main
                moveq   #FIRE_C,d1
                btst.l  d1,d0
                bne     main
                moveq   #FIRE_B,d1
                btst.l  d1,d0
                bne     main
.ww:
.ball:          move.w  xrot,d7
                add.w   d7,rotx
                move.w  yrot,d7
                add.w   d7,roty
                move.w  zrot,d7
                add.w   d7,rotz
                bsr     ball_code
                bsr     wait_int
                add.w   #1,keyloop
                bra     inst_loop

control_ent:    lea     control_txt,a0
                bsr     clear_itxt_area
                bsr     print_string
                move.w  #0,keyloop
                move.w  #0,d7
                move.w  d7,xrot
                move.w  d7,yrot
                move.w  d7,zrot
                move.w  d7,rotx
                move.w  d7,roty
                move.w  d7,rotz
                lea     point_table,a0
                move.l  a0,ptable
control_loop:
                bsr     readpad
                move.l  joycur,d0
                cmp.w   #5,keyloop
                blt     .test3
                move.w  #0,keyloop
                bsr     readpad
                move.l  joycur,d0
                moveq   #FIRE_A,d1
                btst.l  d1,d0
                bne     main
                moveq   #FIRE_C,d1
                btst.l  d1,d0
                bne     main
                moveq   #FIRE_B,d1
                btst.l  d1,d0
                bne     main
                moveq   #JOY_UP,d1
                btst.l  d1,d0
                beq     .testd
                cmp.w   #359,rotspeed
                beq     .testd
                add.w   #1,rotspeed
.testd:         moveq   #JOY_DOWN,d1
                btst.l  d1,d0
                beq     .test1
                cmp.w   #1,rotspeed
                beq     .test1
                sub.w   #1,rotspeed
.test1:         moveq   #KEY_1,d1
                btst.l  d1,d0
                beq     .test2
                lea     point_table,a0
                move.l  a0,ptable
.test2:         moveq   #KEY_2,d1
                btst.l  d1,d0
                beq     .test3
                lea     point_table2,a0
                move.l  a0,ptable
.test3:         moveq   #KEY_3,d1
                btst.l  d1,d0
                beq     .test4
                lea     point_table3,a0
                move.l  a0,ptable
.test4:         moveq   #KEY_4,d1
                btst.l  d1,d0
                beq     .test5
                move.w  rotspeed,d7
                add.w   d7,rotx
.test5:         moveq   #KEY_5,d1
                btst.l  d1,d0
                beq     .test6
                move.w  rotspeed,d7
                add.w   d7,roty
.test6:         moveq   #KEY_6,d1
                btst.l  d1,d0
                beq     .test7
                move.w  rotspeed,d7
                add.w   d7,rotz
.test7:         moveq   #KEY_7,d1
                btst.l  d1,d0
                beq     .test8
                move.w  rotspeed,d7
                sub.w   d7,rotx
.test8:         moveq   #KEY_8,d1
                btst.l  d1,d0
                beq     .test9
                move.w  rotspeed,d7
                sub.w   d7,roty
.test9:         moveq   #KEY_9,d1
                btst.l  d1,d0
                beq     .test0
                move.w  rotspeed,d7
                sub.w   d7,rotz
.test0:         moveq   #KEY_0,d1
                btst.l  d1,d0
                beq     .ww
                move.w  #0,d7
                move.w  d7,xrot
                move.w  d7,yrot
                move.w  d7,zrot
                move.w  d7,rotx
                move.w  d7,roty
                move.w  d7,rotz
                move.w  #1,rotspeed
.ww:
.ball:          bsr     ball_code
                bsr     wait_int
                add.w   #1,keyloop
                bra     control_loop

keyloop:       .dc.w    0
xrot:          .dc.w    0
yrot:          .dc.w    0
zrot:          .dc.w    0
rotspeed:      .dc.w    0
threepointonefour: .dc.w 0

ball_code:
Make_it_spin:
        cmp.w         #0,rotz
        bge           .n
        add.w         #360,rotz
        bra           .next
.n:     cmp.w         #360,rotz
        blt           .next
        sub.w         #360,rotz
.next:
        cmp.w         #0,rotx
        bge           .n2
        add.w         #360,rotx
        bra           .next2
.n2:    cmp.w         #360,rotx
        blt           .next2
        sub.w         #360,rotx
.next2:
        cmp.w         #0,roty
        bge           .n3
        add.w         #360,roty
        bra           .x
.n3:    cmp.w         #360,roty
        blt           .x
        sub.w         #360,roty
.x:     move.w        #160,centx
        move.w        #110,centy
        move.w        #120,centz
;       move.l        #$66666666,BG
        bsr           makerot
;       move.l        #$00000000,BG
dopoints:
        lea           sorted_points,a6
        lea           ball_obj,a4
        move.l        ptable,a0
        moveq.l       #0,d0
        moveq.l       #0,d1
        moveq.l       #0,d2
        moveq.l       #0,d3
        moveq.l       #0,d6
.loop:  move.w        (a0)+,d0
        cmp.w         #$ffff,d0
        beq           .end
        move.w        (a0)+,d1
        move.w        (a0)+,d2
        move.w        (a0)+,d3
        addq.l        #1,d6
        bsr           rotate_axis
        ext.l         d0
        ext.l         d1
        add.l         #30,d1
        ext.l         d2
        ext.l         d3
        move.l        d2,(a6)+
        move.l        d0,(a6)+
        move.l        d1,(a6)+
        move.l        d3,(a6)+
        bra           .loop
.end:
        move.l        #$ffffffff,(a6)+
        bsr           run_gpucode
;       bsr           draw_sorted_points
;       move.l        #$22222222,BG
        move.w        #24,d7
.waste: dbf           d7,.waste
;       move.l        #$0,BG
        rts

                .dc.l "DEAD"
password:
                bsr     clear_itxt_area
;               clr.w   passwd_pos
;               lea     passwd_asc,a0
;               moveq   #7,d0
;clr:           clr.b   (a0)+
;               dbf     d0,.clr
                lea     passwd_txt,a0
                bsr     print_string

                cmp.w   #0,passwd_pos
                beq     .a
                clr.w   d7
                lea     passwd_asc,a6
                move.w  passwd_pos,d6
                move.b  (a6,d6.w),d7
                bra.s   print_passwd
.a:             move.w  #'A',d7
print_passwd:   lea     passwd_asc,a6
                move.w  passwd_pos,d6
                move.b  d7,(a6,d6.w)
                move.b  #' ',1(a6,d6.w)
                clr.b   8(a6)
                lea     -3(a6),a0
                move.w  d7,-(a7)
                bsr     print_string
                move.w  (a7)+,d7
.waitkey:       move.w  d7,-(a7)
                move.l  a6,-(a7)
.ww:            bsr     readpad
.ball1:         move.w  xrot,d7
                add.w   d7,rotx
                move.w  yrot,d7
                add.w   d7,roty
                move.w  zrot,d7
                add.w   d7,rotz
                bsr     ball_code
                bsr     wait_int
                tst.l   joycur
                bne.s   .ww
.ball2:         move.w  xrot,d7
                add.w   d7,rotx
                move.w  yrot,d7
                add.w   d7,roty
                move.w  zrot,d7
                add.w   d7,rotz
                bsr     ball_code
                bsr     wait_int
                bsr     readpad
                move.l  (a7)+,a6
                move.w  (a7)+,d7
                move.l  joycur,d0
                btst.l  #FIRE_A,d0
                bne     validate
                btst.l  #FIRE_B,d0
                bne     validate
                btst.l  #FIRE_C,d0
                bne     validate
                btst.l  #JOY_RIGHT,d0
                bne     pwselect
                btst.l  #JOY_LEFT,d0
                bne     pwdelete
                btst.l  #JOY_UP,d0
                bne     pwup
                btst.l  #JOY_DOWN,d0
                bne     pwdown

                bra     .waitkey
pwdelete:       move.w  passwd_pos,d0
                beq.s   .ex
                subq.w  #1,d0
                move.w  d0,passwd_pos
                move.b  (a6,d0.w),d7
.ex:            bra     print_passwd
pwselect:       move.w  passwd_pos,d0
                cmp.w   #7,d0
                beq.s   .ex
                addq.w  #1,d0
                move.w  d0,passwd_pos
.ex:            bra     print_passwd
pwdown:         subq.w  #1,d7
                bra.s   pwdn2
pwup:           addq.w  #1,d7
pwdn2:          cmp.b   #' '-1,d7
                bne.s   .ok
                move.b  #'~',d7
.ok:            cmp.b   #'a',d7
                bne.s   .ok1
                move.b  #'{',d7
.ok1:           cmp.b   #$7f,d7
                bne.s   .ok2
                move.b  #' ',d7
.ok2:           bra     print_passwd


validate:      ;bsr     readpad
               ;move.l  joycur,d0
               ;bne.s   validate
                move.w  #7,d0
                move.l  a6,a5
                move.w  passwd_pos,d1
                lea     1(a5,d1.w),a5
                sub.w   d1,d0
.clr:           clr.b   (a5)+
                dbf     d0,.clr
                move.b  (a6)+,d4
                lsl.l   #8,d4
                move.b  (a6)+,d4
                lsl.l   #8,d4
                move.b  (a6)+,d4
                lsl.l   #8,d4
                move.b  (a6)+,d4
                move.b  (a6)+,d5
                lsl.l   #8,d5
                move.b  (a6)+,d5
                lsl.l   #8,d5
                move.b  (a6)+,d5
                lsl.l   #8,d5
                move.b  (a6)+,d5
                cmp.l   #'ORIG',d4
                bne.s   .dmo
                cmp.l   #'INAL',d5
                bne.s   .dmo
                move.w  #2,level_file
                clr.w   START_LEVEL

                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp12_end-samp12,d0   ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #20,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp12,a0               ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1

                bra     .ex
.dmo:           cmp.l   #'DEMO',d4
                bne.s   .std
                move.w  #1,level_file
                clr.w   START_LEVEL

                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp11_end-samp11,d0   ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #20,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp11,a0               ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1

                bra.s   .ex
.std:           cmp.l   #'STAN',d4
                bne.s   .skp
                cmp.l   #'DARD',d5
                bne.s   .skp
                move.w  #0,level_file
                clr.w   START_LEVEL

                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp10_end-samp10,d0   ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #20,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp10,a0               ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1

                bra.s   .ex
.skp:           bsr     find_password
.ex:            bra     main

find_password:  move.w  #0,d1
.loop:          move.w  d1,d0
                lsl.w   #5,d0
                lea     level_space+16,a0
                lea     (a0,d0.w),a0
                lea     12(a0),a3
                move.l  4(a0),a1               ;PTR to Nasty Data
                lea     -12(a1),a1             ;Now Points to the Password
                move.l  (a1)+,d6
                eor.l   #$895623ff,d6
                move.l  (a1)+,d7
                eor.l   #$14567821,d7
                tst.l   d6
                beq.s   .next
                cmp.l   d4,d6
                bne.s   .next
                cmp.l   d5,d7
                bne.s   .next
                move.w  d1,START_LEVEL
                bra.s   .ex
.next:          addq.w  #1,d1
                cmp.w   no_of_levels,d1
                blt.s   .loop
                lea     blank_text,a0
                move.b  #8,(a0)+
                move.b  #160,(a0)+
                move.b  #2,(a0)+
                move.b  #'N',(a0)+
                move.l  #'o Le',(a0)+
                move.l  #'vel ',(a0)+
                move.l  #'Foun',(a0)+
                move.w  #'d.',(a0)+
                move.w  #0,(a0)+
                lea     blank_text,a0
                bra     .print
.ex:
                lea     blank_text,a0
                move.b  #8,(a0)+
                move.b  #160,(a0)+
                move.b  #2,(a0)+
                move.b  #'L',(a0)+
                move.l  #'evel',(a0)+
                move.b  #32,(a0)+
.levname:       move.b  (a3)+,d7
                cmp.b   #0,d7
                beq     .levend
                move.b  d7,(a0)+
                bra     .levname
.levend:        move.b  #32,(a0)+
                move.b  #'F',(a0)+
                move.b  #'o',(a0)+
                move.b  #'u',(a0)+
                move.b  #'n',(a0)+
                move.b  #'d',(a0)+
                move.b  #'.',(a0)+
                move.b  #0,(a0)+
                move.b  #0,(a0)+
                lea     blank_text,a0
.print:         bsr     print_string
                move.l  #55,d7
.w1:            jsr     wait_int                ; wait vbl
                dbf     d7,.w1
.waitkey:       bsr     readpad
                move.l  joycur,d0
                btst.l  #FIRE_A,d0
                bne     .b1
.b1:            btst.l  #FIRE_B,d0
                bne     .c
.c:             btst.l  #FIRE_C,d0
                bne     .waitkey
.waitkey2:
                move.l  d6,-(a7)
                move.w  xrot,d7
                add.w   d7,rotx
                move.w  yrot,d7
                add.w   d7,roty
                move.w  zrot,d7
                add.w   d7,rotz
                bsr     ball_code
                bsr     wait_int
                move.l  (a7)+,d6
                bsr     readpad
                move.l  joycur,d0
                cmp.l   d6,d0
                beq     .waitkey2
                rts

i_reset:        btst.l  #KEY_STAR,d0
                beq     intro_loop
                moveq   #2,d0
                bra     exit

start_game:     moveq   #1,d0
                bra     exit

go_slam:        moveq   #-1,d0

exit:           move.w  d0,-(a7)
                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     min_objlist
                move.l  scr_obj,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  #ball_obj,a4
                move.w  #39,d7
.loop:          move.l  (a4)+,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                dbf     d7,.loop
                move.w  (a7)+,d0
                rts

clear_itxt_area:
                lea     screen_32k,a1
                lea     64*160(a1),a1
                move.w  #(128*4)-1,d0
.clr:           clr.l   (a1)+
                clr.l   (a1)+
                clr.l   (a1)+
                clr.l   (a1)+
                clr.l   (a1)+
                clr.l   (a1)+
                clr.l   (a1)+
                clr.l   (a1)+
                clr.l   (a1)+
                clr.l   (a1)+
                dbf     d0,.clr
                rts

print_string:   lea     screen_32k,a1
                moveq   #0,d0
                move.b  (a0)+,d0
                and.w   #$fffc,d0
                lea     (a1,d0.w),a1
                moveq   #0,d0
                move.b  (a0)+,d0
                move.w  d0,d1
                lsl.w   #7,d0
                lsl.w   #5,d1
                add.w   d1,d0
                lea     (a1,d0.w),a1
                move.b  (a0)+,d0
                ext.w   d0
                lsl.w   #2,d0
                move.l  .andoffs(pc,d0.w),d1

.nxt_char:      moveq   #0,d0
                lea     font,a2
                move.b  (a0)+,d0
                beq.s   .ex
                bmi.s   print_string
                sub.w   #$20,d0
                lsl.w   #5,d0
                lea     (a2,d0.w),a2

                movem.l (a2)+,d4-d7
                and.l   d1,d4
                and.l   d1,d5
                and.l   d1,d6
                and.l   d1,d7
                move.l  d4,(a1)
                move.l  d5,160(a1)
                move.l  d6,320(a1)
                move.l  d7,480(a1)
                movem.l (a2)+,d4-d7
                and.l   d1,d4
                and.l   d1,d5
                and.l   d1,d6
                and.l   d1,d7
                move.l  d4,640(a1)
                move.l  d5,800(a1)
                move.l  d6,960(a1)
                move.l  d7,1120(a1)
                addq.l  #4,a1
                bra.s   .nxt_char
.ex:            rts

.andoffs:
                .dc.b   0,0,0,0
                .dc.b   $11,$11,$11,$11
                .dc.b   $22,$22,$22,$22
                .dc.b   $33,$33,$33,$33
                .dc.b   $44,$44,$44,$44
                .dc.b   $55,$55,$55,$55
                .dc.b   $66,$66,$66,$66
                .dc.b   $77,$77,$77,$77
                .dc.b   $88,$88,$88,$88
                .dc.b   $99,$99,$99,$99
                .dc.b   $aa,$aa,$aa,$aa
                .dc.b   $bb,$bb,$bb,$bb
                .dc.b   $cc,$cc,$cc,$cc
                .dc.b   $dd,$dd,$dd,$dd
                .dc.b   $ee,$ee,$ee,$ee
                .dc.b   $ff,$ff,$ff,$ff

lower_logo:     moveq   #63,d0
                lea     gamelogo,a0
                lea     screen_32k,a1
.lp:            movem.l d0/a0-a1,-(a7)
                move.w  d0,d1
                lsl.w   #7,d0
                lsl.w   #5,d1
                add.w   d1,d0
                lea     (a0,d0.w),a0
.more:          movem.l (a0)+,d1-d7/a2-a4
                movem.l d1-d7/a2-a4,(a1)
                movem.l (a0)+,d1-d7/a2-a4
                movem.l d1-d7/a2-a4,40(a1)
                movem.l (a0)+,d1-d7/a2-a4
                movem.l d1-d7/a2-a4,80(a1)
                movem.l (a0)+,d1-d7/a2-a4
                movem.l d1-d7/a2-a4,120(a1)
                lea     160(a1),a1
                cmp.l   #gamelogo_end,a0
                bne.s   .more
                bsr     wait_int
                movem.l (a7)+,d0/a0-a1
                dbf     d0,.lp
                rts

raise_cr:       moveq   #7,d0
                lea     cpyrt,a0
                lea     screen_32k,a1
                add.l   #32000,a1
.lp:            movem.l d0/a0-a1,-(a7)
                moveq   #8,d1
                sub.w   d0,d1
                move.w  d1,d0
                lsl.w   #7,d0
                lsl.w   #5,d1
                add.w   d1,d0
                lea     (a0,d0.w),a0
.more:          movem.l 120(a0),d1-d7/a2-a4
                movem.l d1-d7/a2-a4,-(a1)
                movem.l 80(a0),d1-d7/a2-a4
                movem.l d1-d7/a2-a4,-(a1)
                movem.l 40(a0),d1-d7/a2-a4
                movem.l d1-d7/a2-a4,-(a1)
                movem.l (a0),d1-d7/a2-a4
                movem.l d1-d7/a2-a4,-(a1)
                lea     -160(a0),a0
                cmp.l   #cpyrt,a0
                bne.s   .more
                bsr     wait_int
                movem.l (a7)+,d0/a0-a1
                dbf     d0,.lp
                rts

load_gpucode:
.test:          move.l  B_CMD,d0                ; make sure blitter
                btst    #0,d0                   ; is done
                beq.s   .test

                lea     start_gpucode,a0       ; start of GPU code
                lea     end_gpucode,a1
                move.w  #end_gpucode-start_gpucode,d6
                move.l  #G_1+$8000,a2
.copy:
                moveq.l #0,d7
                move.l  a0,A2_BASE
                move.l  a2,A1_BASE
                move.l  d7,A1_PIXEL
                move.l  d7,A2_PIXEL
                move.w  #1,d0
                swap    d0
                move.w  #1024,d0
                move.l  d0,B_COUNT              ;$00010001 = 4 Bytes
                move.l  #PIXEL32|XADDPHR,d6
                move.l  d6,A1_FLAGS
                move.l  d6,A2_FLAGS
                move.l  #SRCEN|LFU_S,d7
                move.l  d7,B_CMD
                move.l  #0,A1_CLIP
                move.w  #8,d7
.loop:          dbf     d7,.loop
                rts

run_gpucode:
.test2:         move.l  B_CMD,d0
                btst    #0,d0
                beq.s   .test2

                move.l  #G_1,G_PC
                move.l  #GPUGO,G_CTRL           ; Start GPU
.wait:
                move.l  G_CTRL,d0               ; Wait for complete
                andi.l  #$1,d0
                bne.s   .wait
                rts

        .dphrase

start_gpucode:

        .gpu

        .org     $f03080

G_1:

                movei   #mystack1,S_P

                movei   #$ffffffff,r6
                movei   #$fffffffe,r7
                movei   #$100000,r9
                movei   #ball_obj,r16
                movei   #sorted_points2,r15
.start:         movei   #sorted_points,r14
                nop
                load    (r14),r4
.loop1:
                load    (r14+4),r5
                nop
                cmp     r7,r5
                beq     .next
                cmp     r6,r5
                beq     .end1
.checkr5:       cmp     r7,r4
                bne     .comp
                bra     .swap
.comp:          cmp     r5,r4
                bgt     .next
                nop
.swap:          move    r5,r10
                move    r4,r5
                move    r10,r4
.next:          addq    #16,r14
                nop
                bra     .loop1
.end1:
                cmp     r7,r4
                beq    .exit
                movei   #sorted_points,r14
                nop
                nop
.loop2:
                load    (r14),r5
                nop
                cmp     r6,r5
                beq    .start
                cmp    r4,r5
                bne    .next2
                movei  #ball_data,r1   ; start of ball data
                movei  #$ffffc007,r18  ; masks
                movei  #$00003ff8,r20
                movei  #$fffff000,r13
                movei  #$000007ff,r17
                movei  #$fffff800,r22
                load   (r14+1),r11     ; set x,y,z
                load   (r14+2),r12
                load   (r14+3),r10

                load   (r16),r15
                shlq   #8,r10          ; balls are 256bytes each
                shlq   #4,r12          ; shift it
                add    r10,r1          ; now pointing at correct ball
                load   (r15),r3
                shlq   #8,r1
                and    r17,r3          ; get rid of old data ptr
                and    r22,r1
                and    r20,r12         ; mask off unnecessary bits
                load   (r15+1),r19
                or     r1,r3           ; insert new data ptr
                movei  #$fff,r20
.y:             and    r18,r19         ; clear old x co-ord
                store  r3,(r15)
                or     r12,r19
                load   (r15+3),r21
                store  r19,(r15+1)
.x:             and    r13,r21         ; clear old y co-ord
                and    r20,r11         ; make sure top is empty
                or     r21,r11         ; plonk it in
                store  r11,(r15+3)
                addq   #4,r16          ; position for next ball

                store  r7,(r14)
                nop
.next2:
                addq   #16,r14         ; next ball
                bra    .loop2
                nop
.exit:          nop
                nop
                moveq   #0,r0          ; Stop GPU
                movei   #G_CTRL,Junk1
                store   r0,(Junk1)
                nop
                nop

        .dphrase
                .dcb.l  24,0
mystack1:       .dc.l   0
        .68000
end_gpucode:


ins_sort:
       lea            sorted_points,a6
       move.l         (a6),d4
.loop1:
       move.l         16(a6),d5
       cmp.l          #-1,d5
       beq.s          .end1
       cmp.l          d5,d4
       bgt.s          .next
       exg            d5,d4
.next: add.l          #16,a6
       bra.s          .loop1
.end1:
       cmp.l          #-2,d4
       beq.s          .exit
       lea            sorted_points,a6
.loop2:
       move.l         (a6),d5
       cmp.l          #-1,d5
       beq.s          .end2
       cmp.l          d4,d5
       bne.s          .next2
       movem.l        (a6),d0-d3
       bsr            plot_sprite
       move.l         #$fffffffe,(a6)
.next2:
       lea            16(a6),a6
       bra            .loop2
.end2: bra            ins_sort
.exit: rts

draw_sorted_points:
       lea            ball_obj,a4
       lea            sorted_points2,a6
.loop: move.l         (a6)+,d0
       cmp.l          #-1,d0
       beq.s          .exit
       move.l         (a6)+,d1
       move.l         (a6)+,d2
       move.l         (a6)+,d3
       bsr            plot_sprite
       bra            .loop
.exit: rts

****************************************************
**  The following variables should be             **
**  set up too ;-                                 **
**  centx, centy, centz     centre of rotation    **
**  rotx, roty, rotz,       angle of rotation     **
**  transx, transy, transz  translation of point  **
****************************************************

makerot:
        movem.l       d0-d7/a0-a6,-(a7)    ; Save registers

matinit:
        move.w        #0,d1                ; Initialise matrix
        move.w        #16384,d2            ; Sin(90)
        move.w        d2,matrix11
        move.w        d1,matrix12          ; 1 0 0
        move.w        d1,matrix13          ; 0 1 0
        move.w        d1,matrix21          ; 0 0 1
        move.w        d2,matrix22
        move.w        d1,matrix23
        move.w        d1,matrix31
        move.w        d1,matrix32
        move.w        d2,matrix33


zrotate:
        move.w        rotz,d0
        beq           yrotate
        jsr           sincos
        move.w        d1,sinz
        move.w        d2,cosz
        move.w        d1,d3
        move.w        d2,d4
        muls          matrix11,d2
        muls          matrix12,d1
        sub.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx11
        move.w        d3,d1
        move.w        d4,d2
        muls          matrix21,d2
        muls          matrix22,d1
        sub.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx21
        move.w        d3,d1
        move.w        d4,d2
        muls          matrix31,d2
        muls          matrix32,d1
        sub.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx31
        move.w        d3,d1
        move.w        d4,d2
        muls          matrix11,d1
        muls          matrix12,d2
        add.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx12
        move.w        d3,d1
        move.w        d4,d2
        muls          matrix21,d1
        muls          matrix22,d2
        add.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx22
        muls          matrix31,d3
        muls          matrix32,d4
        add.l         d3,d4
        lsl.l         #2,d4
        swap          d4
        move.w        d4,rotx32
        move.w        matrix13,rotx13
        move.w        matrix23,rotx23
        move.w        matrix33,rotx33
        move.l        #8,d7
        move.l        #rotx11,a1
        move.l        #matrix11,a2

zrotlop1:
        move.w        (a1)+,(a2)+
        dbra          d7,zrotlop1

yrotate:
        move.w        roty,d0
        beq           xrotate
        jsr           sincos
        move.w        d1,siny
        move.w        d2,cosy
        move.w        d1,d3
        move.w        d2,d4
        muls          matrix11,d2
        muls          matrix13,d1
        add.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx11
        move.w        d3,d1
        move.w        d4,d2
        muls          matrix21,d2
        muls          matrix23,d1
        add.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx21
        move.w        d3,d1
        move.w        d4,d2
        muls          matrix31,d2
        muls          matrix33,d1
        add.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx31
        neg.w         d3
        move.w        d3,d1
        move.w        d4,d2
        move.w        matrix12,rotx12
        move.w        matrix22,rotx22
        move.w        matrix32,rotx32
        muls          matrix11,d1
        muls          matrix13,d2
        add.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx13
        move.w        d3,d1
        move.w        d4,d2
        muls          matrix21,d1
        muls          matrix23,d2
        add.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx23
        muls          matrix31,d3
        muls          matrix33,d4
        add.l         d3,d4
        lsl.l         #2,d4
        swap          d4
        move.w        d4,rotx33
        move.l        #8,d7
        move.l        #rotx11,a1
        move.l        #matrix11,a2

yrotlop1:
        move.w        (a1)+,(a2)+
        dbra          d7,yrotlop1

xrotate:
        move.w        rotx,d0
        beq           makerotend
        jsr           sincos
        move.w        d1,sinx
        move.w        d2,cosx
        move.w        d1,d3
        move.w        d2,d4
        move.w        matrix11,rotx11
        move.w        matrix21,rotx21
        move.w        matrix31,rotx31
        muls          matrix12,d2
        muls          matrix13,d1
        sub.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx12
        move.w        d3,d1
        move.w        d4,d2
        muls          matrix22,d2
        muls          matrix23,d1
        sub.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx22
        move.w        d3,d1
        move.w        d4,d2
        muls          matrix32,d2
        muls          matrix33,d1
        sub.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx32
        move.w        d3,d1
        move.w        d4,d2
        muls          matrix12,d1
        muls          matrix13,d2
        add.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx13
        move.w        d3,d1
        move.w        d4,d2
        muls          matrix22,d1
        muls          matrix23,d2
        add.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        move.w        d2,rotx23
        muls          matrix32,d3
        muls          matrix33,d4
        add.l         d3,d4
        lsl.l         #2,d4
        swap          d4
        move.w        d4,rotx33
        move.l        #rotx11,a1
        move.l        #matrix11,a2
        move.l        #9,d7
        subq.l        #1,d7

rotxlop1:
        move.w        (a1)+,(a2)+
        dbra          d7,rotxlop1
makerotend:
        movem.l       (a7)+,d0-d7/a0-a6    ; Restore Registers
        rts

****************************************************
**  Pass X,Y and Z coords of point to be rotated  **
**  in d0-d2, the rotated values are returned in  **
**  d0-d2 also. Ensure that makerot has been done **
**  first !!!                                     **
****************************************************

rotate_axis:
        movem.l       d3-d7/a0-a6,-(a7)    ; Save out registers
        sub.w         centx,d0             ; Add Pre- offset
        sub.w         centy,d1
        sub.w         centz,d2
        move.w        d0,d4                ; Back up X,Y,Z coord        s
        move.w        d1,d5
        move.w        d2,d6

        muls          matrix11,d0          ; Calculate new X coord
        muls          matrix21,d1
        muls          matrix31,d2
        add.l         d0,d1
        add.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        add.w         centx,d2             ; Add Post- offset
        add.w         transx,d2            ; Apply X translate
        move.w        d2,-(a7)             ; Stack new X

        move.w        d4,d0                ; Reload X,Y,Z values
        move.w        d5,d1
        move.w        d6,d2

        muls          matrix12,d0          ; Matrix mult for new Y
        muls          matrix22,d1
        muls          matrix32,d2
        add.l         d0,d1
        add.l         d1,d2
        lsl.l         #2,d2
        swap          d2
        add.w         centy,d2             ; Add Post- offset
        add.w         transy,d2            ; Apply Y translate
        move.w        d2,-(a7)             ; Stack new Y

        muls          matrix13,d4          ; Matrix mult for new Z
        muls          matrix23,d5
        muls          matrix33,d6
        add.l         d4,d5
        add.l         d5,d6
        lsl.l         #2,d6
        swap          d6
        add.w         centz,d6             ; Add Post- offset
        add.w         transz,d6            ; Apply Z translate
        move.w        d6,d2                ;
        move.w        (a7)+,d1             ; Restore Y and X from sta   ck
        move.w        (a7)+,d0

        movem.l       (a7)+,d3-d7/a0-a6    ; Pull back registers
        rts


sincos: tst.w         d0
        bpl           noaddi
        add.w         #360,d0

noaddi: move.l        #sintab,a1
        move.l        d0,d2
        lsl.w         #1,d0
        move.w        0(a1,d0.w),d1
        cmp.w         #270,d2
        blt           plus9
        sub.w         #270,d2
        bra           sendsin

plus9:  add.w         #90,d2

sendsin:
        lsl.w         #1,d2
        move.w        0(a1,d2.w),d2
        rts


plot_sprite:
;       add.w         #50,d1
;       add.w         #60,d2
        move.l        (a4),a3
        lea           ball_data,a5     ; start of ball data
        lsl.l         #8,d3            ; balls are 256bytes each
        add.l         d3,a5            ; now pointing at correct ball
        move.l        a5,d7            ; copy to data reg
        lsl.l         #8,d7
        and.l         #$7ff,(a3)       ; get rid of old data ptr
        or.l          d7,(a3)          ; insert new data ptr
.y:     and.w         #$c007,6(a3)     ; clear old x co-ord
        lsl.l         #4,d2            ; shift it
        and.w         #$3ff8,d2        ; mask off unnecessary bits
        or.w          d2,6(a3)         ; plonk it in
.x:     and.w         #$f000,14(a3)    ; clear old y co-ord
        and.w         #$fff,d1         ; make sure top is empty
        or.w          d1,14(a3)        ; plonk it in
        add.l         #4,a4            ; position for next ball
        rts

                .phrase

game:           sf      HA_HA_NO_MORE_PLAY_TODAY
.next:          bsr     INITIALISE_GAME_DATA
                bsr     add_game_objs
                bsr     wait_int
                bsr     wait_int
                lea     blank_pal,a0
                bsr     set_palette16
                bsr     max_objlist

GAME_LOOP:      clr.w   erase_pixel_flag
                clr.w   player_dead
                sf      skip_level_flag
                sf      totally_dead_dude
TF1:            move.w  time_loop_value,TIME_FRAME
                bsr     hide_sprites
                bsr     draw_level

                move.w  #2,me_dir
.start_mv_samp: movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp18_end-samp18,d0   ;sample length
                move.l  d0,d1                   ;looping length
                moveq   #25,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp18,a0               ;start address
                move.l  a0,a1                   ;loop start address
                jsr     player+$c
                move.w  d0,movesamp
                movem.l (a7)+,d0-d4/a0-a1

MAIN_LOOP:      bsr     wait_int
                bsr     fill_logical_complete_box
                bsr     key_check
                tst.w   paused
                bne     pause_loop
main_cont:      bsr     test_me_direction
                bsr     check_for_complete_box
                bsr     move_sprites
                bsr     erase_pixels
                bsr     update_hole_table
                bsr     move_nasties
                bsr     is_player_dead
                bsr     next_level_yet
                tst.w   totally_dead_dude
                bne.s   .exit
                bsr     draw_player
                bsr     draw_nasties
                bsr     OCCASIONAL_STUFF
                tst.w   skip_level_flag
                bne     LEVEL_COMPLETED
                tst.b   K_A
                beq     MAIN_LOOP
                bsr     DISPLAY_GAME_ABORTED
                bra     .exit_completely
.exit:          tst.b   HA_HA_NO_MORE_PLAY_TODAY
                beq.s   .skip
                tst.w   final_levels
                bne.s   .dispend
.skip:          bsr     display_game_over
.exit_completely:

                bsr     min_objlist
                clr.l   vbl
                bsr     del_game_objs
                rts
.dispend:
                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .next
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
.next:
                bsr     del_game_obj1
                bsr     DISPLAY_END_OF_PROGGY
                bsr     min_objlist
                bsr     del_game_obj2
                clr.l   vbl
                rts

hide_sprites:   tst.l   player_obj
                beq.s   .ex
                move.w  #3000,d0
                moveq   #0,d1
                move.l  player_obj,a0
                move.l  #blank_pal,a1
                bsr     draw_sprite
                move.l  nasty_obj1,a0
                bsr     draw_sprite
                move.l  nasty_obj2,a0
                bsr     draw_sprite
                move.l  nasty_obj3,a0
                bsr     draw_sprite
                move.l  nasty_obj4,a0
                bsr     draw_sprite
                move.l  nasty_obj5,a0
                bsr     draw_sprite
.ex:            rts

del_game_objs:  bsr     wait_int
                move.l  player_obj,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  nasty_obj1,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  nasty_obj2,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  nasty_obj3,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  nasty_obj4,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  nasty_obj5,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  scr_obj,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                clr.l   player_obj
                rts

del_game_obj1:  bsr     wait_int
                move.l  player_obj,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  nasty_obj1,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  nasty_obj2,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  nasty_obj3,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  nasty_obj4,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                move.l  nasty_obj5,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                rts
del_game_obj2:  move.l  scr_obj,d0
                sub.l   #shadow_base,d0
                bsr     del_object
                clr.l   player_obj
                rts

add_game_objs:  moveq   #2,d0
                moveq   #-1,d1
                bsr     add_object
                add.l   #shadow_base,d7
                move.l  d7,scr_obj
                moveq   #3,d0
                moveq   #-1,d1
                bsr     add_object
                add.l   #shadow_base,d7
                move.l  d7,player_obj
                moveq   #3,d0
                moveq   #-1,d1
                bsr     add_object
                add.l   #shadow_base,d7
                move.l  d7,nasty_obj1
                moveq   #3,d0
                moveq   #-1,d1
                bsr     add_object
                add.l   #shadow_base,d7
                move.l  d7,nasty_obj2
                moveq   #3,d0
                moveq   #-1,d1
                bsr     add_object
                add.l   #shadow_base,d7
                move.l  d7,nasty_obj3
                moveq   #3,d0
                moveq   #-1,d1
                bsr     add_object
                add.l   #shadow_base,d7
                move.l  d7,nasty_obj4
                moveq   #3,d0
                moveq   #-1,d1
                bsr     add_object
                add.l   #shadow_base,d7
                move.l  d7,nasty_obj5
                rts
med_rez:        bsr     hide_sprites
                bsr     clear_screen
                bsr     wait_int
                move.w  #%001011000111,VMODE    ; 2C7 Med Res 16bit
                rts
low_rez:        bsr     hide_sprites
                bsr     clear_screen
                bsr     del_fnt_objs
                move.w  #%011011000111,VMODE    ; 6C7 Low Res 16bit
                rts


key_check:      bsr     readpad
                tst.b   control
                beq.s   .orig
                bsr     new_control
                bra.s   .skip_orig
.orig:          move.l  joycur,d0
                btst.l  #JOY_UP,d0
                sne     K_U
                btst.l  #JOY_DOWN,d0
                sne     K_D
                btst.l  #JOY_LEFT,d0
                sne     K_L
                btst.l  #JOY_RIGHT,d0
                sne     K_R
.skip_orig:     btst.l  #KEY_3,d0
                sne     K_3
                btst.l  #KEY_1,d0
                sne     K_1
                sf      K_A
                btst.l  #KEY_HASH,d0
                beq.s   .next
                btst.l  #KEY_STAR,d0
                sne     K_A
.next:          btst.l  #FIRE_B,d0
                beq.s   .no_fire
                sf      K_F
                tst.b   K_FR
                bne.s   .nxtk
                st      K_F
                st      K_FR
                bra.s   .nxtk
.no_fire:       sf      K_FR
.nxtk:          btst.l  #PAUSE,d0
                beq.b   .6
                tst.w   pawslock
                bne.s   .exit
                eor.w   #1,paused
                move.w  #1,pawslock
                bra     .exit
.6:             move.w  #0,pawslock
.exit:          rts

paused:         .dc.w   0
pawslock:       .dc.w   0

new_control:    sf      K_U
                sf      K_D
                sf      K_L
                sf      K_R
                bsr     re_set
                move.l  joycur,d0
                btst.l  #JOY_UP,d0
                beq.s   .skip_up
                st      K_U
                sf      K_D
.skip_up:       btst.l  #JOY_DOWN,d0
                beq.s   .skip_down
                st      K_D
                sf      K_U
.skip_down:     btst.l  #JOY_LEFT,d0
                beq.s   .skip_left
                st      K_L
                sf      K_R
.skip_left:     btst.l  #JOY_RIGHT,d0
                beq.s   .skip_right
                st      K_R
                sf      K_L
.skip_right:    rts
re_set:         move.w  me_dir,d0
                lsl.w   #2,d0
                move.l  dir_tab(pc,d0.w),a0
                st      (a0)
                rts
dir_tab:        .dc.l   K_U,K_D,K_R,K_L

pause_loop:     movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp9_end-samp9,d0     ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #20,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp9,a0                ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                move.l  #75,d1
.wait:          jsr     wait_int                ; wait vbl
                dbf     d1,.wait
                movem.l (a7)+,d0-d4/a0-a1

                move.w  #0,pause_opt
                moveq   #7,d0                   ; paused
                moveq   #-1,d1                  ; end of object list
                move.w  #70,d2
                move.w  #100,d3
                move.l  #pause_spr,d4
                moveq   #0,d5
                jsr     add_spr                 ; call add_object
                move.l  d7,pause_adds           ; store address of object

                lea     dsp_chan1,a0            ;point to dsp table
                lea     store_channels,a1       ;store buffer
                move.w  #31,d0                  ;32 * 8
.cop_loop1:     move.l  (a0)+,(a1)+
                move.l  (a0)+,(a1)+
                dbf     d0,.cop_loop1
                lea     samp_flag,a0
                move.l  (a0)+,(a1)+
                move.l  (a0)+,(a1)+
                move.w  #0,d0
                jsr     player+$14

.loop:          jsr     readpad                 ; read in the controller status
                jsr     key_check               ; respond to controller input
                tst.w   paused                  ; is paused cleared
                beq.s   .exit                   ; yes
                jsr     do_joy
                jsr     wait_int                ; wait vbl
                cmp.w   #2,pause_opt
                beq.s   .loop
                moveq   #5,d0
                moveq   #4,d1
                jsr     player+$10              ;clear all 4 efx channels
                bra     .loop                   ;round the loop
.exit:          cmp.w   #3,pause_opt
                beq.s   .over1
                bsr     clear_sprs
.over1:         moveq   #5,d0
                moveq   #4,d1
                jsr     player+$10              ;clear all 4 efx channels

                lea     dsp_chan1,a0            ;point to dsp table
                lea     store_channels,a1       ;store buffer
                move.w  #31,d0                  ;32 * 8
.cop_loop2:     move.l  (a1)+,(a0)+
                move.l  (a1)+,(a0)+
                dbf     d0,.cop_loop2
                lea     samp_flag,a0
                move.l  (a1)+,(a0)+
                move.l  (a1)+,(a0)+

                moveq   #3,d7                   ;4 sample channels
                move.w  effects_vol,d1
                moveq   #0,d2                   ;leave freq alone
.vol_loop1:     moveq   #8,d0
                sub.l   d7,d0
                jsr     player+$24
                dbf     d7,.vol_loop1

                move.w  music_vol,d0
                jsr     player+$14
                bra     main_cont

do_joy:         move.l  joycur,d0
                btst.l  #FIRE_A,d0
                beq.s   .over1
                cmp.w   #1,pause_opt
                beq     .exit
                bsr     plot_music_spr
                lea     mod_table,a1            ;play curr mod
                move.w  curr_mod,d7
                lsl.l   #2,d7
                add.l   d7,a1
                move.l  (a1),a0
                jsr     player+4                ;init new mod
                move.w  #1,pause_opt
                bra     .exit
.over1:         btst.l  #FIRE_B,d0
                beq.s   .over2
                cmp.w   #2,pause_opt
                beq     .exit
                bsr     plot_effects_spr
                movem.l d0-d4/a0-a1,-(a7)
                move.w  #5,d0
                move.w  #4,d1
                jsr     player+$10              ;Turn off all samples
                move.l  #samp2_end-samp2,d0     ;sample length
                move.l  d0,d1                   ;looping length
                moveq   #20,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp2,a0                ;start address
                move.l  a0,a1                   ;loop start address
                jsr     player+$c
                move.w  d0,pausesamp
                move.w  #2,psamp
                movem.l (a7)+,d0-d4/a0-a1
                move.w  #2,pause_opt
                bra     .exit
.over2:         btst.l  #JOY_UP,d0
                beq     .notup
                cmp.w   #0,uplock
                bne     .over2b
                move.w  #1,uplock
                cmp.w   #2,pause_opt
                bne     .startmus
                move.l  d0,-(a7)
                move.l  d1,-(a7)
                move.w  pausesamp,d0
                cmp.w   #0,d0
                beq.s   .start
                move.w  #1,d1
                jsr     player+$10              ;Turn off pause sample
                clr.w   pausesamp
                move.l  (a7)+,d1
                move.l  (a7)+,d0
.start:
                move.w  psamp,d7
                add.w   #1,d7
                cmp.w   #20,d7
                bne.s   .chg
                move.w  #2,d7
.chg:           move.w  d7,psamp
                sub.w   #1,d7
                lsl.w   #3,d7
                movem.l d0-d4/a0-a1,-(a7)
                lea     samp_table,a6
                move.l  (a6,d7.w),d0
                move.l  #0,d1                    ;looping length
                moveq   #20,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                move.l  4(a6,d7.w),a0           ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                move.w  d0,pausesamp
                movem.l (a7)+,d0-d4/a0-a1
                bra.s   .over2b
.startmus:
                add.w   #1,curr_mod
                cmp.w   #12,curr_mod
                bne.s   .play
                move.w  #0,curr_mod
.play:          lea     mod_table,a1
                move.w  curr_mod,d7
                lsl.l   #2,d7
                add.l   d7,a1
                move.l  (a1),a0
                jsr     player+4                ;init new mod
                bra.s   .over2b

.notup:         move.w  #0,uplock
.over2b:        btst.l  #JOY_DOWN,d0
                beq     .notdn
                cmp.w   #0,dnlock
                bne     .over2c
                move.w  #1,dnlock
                cmp.w   #2,pause_opt
                bne     .startmus2
                move.l  d0,-(a7)
                move.l  d1,-(a7)
                move.w  pausesamp,d0
                cmp.w   #0,d0
                beq.s   .start2
                move.w  #1,d1
                jsr     player+$10              ;Turn off pause sample
                clr.w   pausesamp
                move.l  (a7)+,d1
                move.l  (a7)+,d0
.start2:
                move.w  psamp,d7
                sub.w   #1,d7
                cmp.w   #1,d7
                bne.s   .chg2
                move.w  #19,d7
.chg2:          move.w  d7,psamp
                sub.w   #1,d7
                lsl.w   #3,d7
                movem.l d0-d4/a0-a1,-(a7)
                lea     samp_table,a6
                move.l  (a6,d7.w),d0
                move.l  #0,d1                   ;looping length
                moveq   #20,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                move.l  4(a6,d7.w),a0           ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                move.w  d0,pausesamp
                movem.l (a7)+,d0-d4/a0-a1
                bra.s   .over2c
.startmus2:
                cmp.w   #0,curr_mod
                bne.s   .play2
                move.w  #12,curr_mod
.play2:         sub.w   #1,curr_mod
                lea     mod_table,a1
                move.w  curr_mod,d7
                lsl.l   #2,d7
                add.l   d7,a1
                move.l  (a1),a0
                jsr     player+4                ;init new mod
                bra.s   .over2c

.notdn:         move.w  #0,dnlock
.over2c:        btst.l  #KEY_1,d0
                bne.s   .clear
                btst.l  #KEY_3,d0
                beq.s   .over3
.clear:         cmp.w   #3,pause_opt
                beq     .exit
                bsr     clear_sprs
                move.w  #3,pause_opt
                bra     .exit
.over3:         btst.l  #JOY_LEFT,d0
                beq.s   .over5
                cmp.w   #1,pause_opt
                bne.s   .over4
                cmp.w   #1,music_vol
                beq     .exit
                jsr     sub_mus_spr
                subq.w  #1,music_vol
                move.w  music_vol,d0
                jsr     player+$14
                bra     .exit
.over4:         cmp.w   #2,pause_opt
                bne     .exit
                cmp.w   #1,effects_vol
                beq     .exit
                jsr     sub_fx_spr
                subq.w  #1,effects_vol
                move.w  effects_vol,d0
                jsr     player+$18
                bra     .exit
.over5:         btst.l  #JOY_RIGHT,d0
                beq.s   .exit
                cmp.w   #1,pause_opt
                bne.s   .over6
                cmp.w   #255,music_vol
                beq     .exit
                jsr     add_mus_spr
                addq.w  #1,music_vol
                move.w  music_vol,d0
                jsr     player+$14
                bra     .exit
.over6:         cmp.w   #2,pause_opt
                bne     .exit
                cmp.w   #255,effects_vol
                beq     .exit
                jsr     add_fx_spr
                addq.w  #1,effects_vol
                move.w  effects_vol,d0
                jsr     player+$18
.exit:          rts

plot_music_spr:
                bsr     clear_sprs
                moveq   #7,d0
                moveq   #-1,d1
                move.w  #70,d2
                move.w  #100,d3
                move.l  #music_spr,d4
                moveq   #0,d5
                jsr     add_spr
                move.l  d7,music_adds
                moveq   #7,d0
                moveq   #-1,d1
                move.w  #70,d2
                move.w  #140,d3
                move.l  #mus_vol_spr,d4
                moveq   #0,d5
                jsr     add_spr
                move.l  d7,red_adds
                moveq   #7,d0
                moveq   #-1,d1
                move.w  #70,d2
                move.w  #140,d3
                move.l  #arrow_spr,d4
                moveq   #1,d5
                jsr     add_spr
                move.l  d7,arrow_adds
                rts

plot_effects_spr:
                bsr     clear_sprs
                moveq   #7,d0
                moveq   #-1,d1
                move.w  #70,d2
                move.w  #100,d3
                move.l  #fx_spr,d4
                moveq   #0,d5
                jsr     add_spr
                move.l  d7,fx_adds
                moveq   #7,d0
                moveq   #-1,d1
                move.w  #70,d2
                move.w  #140,d3
                move.l  #fx_vol_spr,d4
                moveq   #0,d5
                jsr     add_spr
                move.l  d7,red_adds
                moveq   #7,d0
                moveq   #-1,d1
                move.w  #70,d2
                move.w  #140,d3
                move.l  #arrow_spr,d4
                moveq   #1,d5
                jsr     add_spr
                move.l  d7,arrow_adds
                rts

clear_sprs:
                cmp.w   #0,pause_opt
                bne.s   .over3
                move.l  pause_adds,d0
                jsr     del_object
                bra.s   .exit
.over3:         cmp.w   #1,pause_opt
                bne.s   .over1
                move.l  music_adds,d0
                jsr     del_object
                bra.s   .over2
.over1:         cmp.w   #2,pause_opt
                bne.s   .exit
                move.l  fx_adds,d0
                jsr     del_object
.over2:         move.l  red_adds,d0
                jsr     del_object
                move.l  arrow_adds,d0
                jsr     del_object
.exit:          rts

add_spr:
                and.l   #$ffff,d2               ;new x pos
                and.l   #$ffff,d3               ;new y pos
                lsl.l   #1,d3                   ;double y pos for objlist
                lsl.l   #3,d3                   ;3 bit shift for objlist format
                move.w  d0,d7                   ;pre def obj number
                lea     pre_def_objects,a6
                lsl.l   #5,d7                   ;mult 32
                add.l   d7,a6
                lsl.l   #8,d4                   ;8 bit shift for objlist format
                and.l   #$ffff7fff,8(a6)
                lsl.l   #8,d5
                lsl.l   #7,d5
                or.l    d5,8(a6)                ;set trans bit
                and.l   #%11111111111111111100000000000111,4(a6)
                or.l    d3,4(a6)
                and.l   #%11111111111111111111000000000000,12(a6)
                or.l    d2,12(a6)
                and.l   #%00000000000000000000011111111111,(a6)
                or.l    d4,(a6)
                jsr     add_object
                rts

sub_mus_spr:
                lea     mus_vol_spr,a0
                add.l   #(220*2),a0
                moveq   #0,d0
                move.w  m_vol,d0
                bsr     sub_v_spr
                sub.w   #217,m_vol
                rts

add_mus_spr:
                add.w   #217,m_vol
                lea     mus_vol_spr,a0
                lea     red_spr,a1
                add.l   #(220*2),a0
                add.l   #(220*2),a1
                moveq   #0,d0
                move.w  m_vol,d0
                bsr     add_v_spr
                rts

sub_fx_spr:
                lea     fx_vol_spr,a0
                add.l   #(220*2),a0
                moveq   #0,d0
                move.w  e_vol,d0
                bsr     sub_v_spr
                sub.w   #217,e_vol
                rts

add_fx_spr:
                add.w   #217,e_vol
                lea     fx_vol_spr,a0
                lea     red_spr,a1
                add.l   #(220*2),a0
                add.l   #(220*2),a1
                moveq   #0,d0
                move.w  e_vol,d0
                bsr     add_v_spr
                rts

sub_v_spr:      lsr.l   #8,d0           ;div 256
                lsl.l   #1,d0           ;and mul 2
                add.l   d0,a0
                move.l  #$00000000,d1   ;colour for blanked volume bar
                bsr     do_spr
                rts

add_v_spr:      lsr.l   #8,d0           ;div 256
                lsl.l   #1,d0           ;and mul 2
                add.l   d0,a0
                add.l   d0,a1
                move.l  (a1),d1
do_spr:         move.l  d1,(a0)
                move.l  d1,220*2*1(a0)
                move.l  d1,220*2*2(a0)
                move.l  d1,220*2*3(a0)
                move.l  d1,220*2*4(a0)
                move.l  d1,220*2*5(a0)
                move.l  d1,220*2*6(a0)
                move.l  d1,220*2*7(a0)
                move.l  d1,220*2*8(a0)
                move.l  d1,220*2*9(a0)
                move.l  d1,220*2*10(a0)
                move.l  d1,220*2*11(a0)
                move.l  d1,220*2*12(a0)
                move.l  d1,220*2*13(a0)
                move.l  d1,220*2*14(a0)
                move.l  d1,220*2*15(a0)
                move.l  d1,220*2*16(a0)
                move.l  d1,220*2*17(a0)
                move.l  d1,220*2*18(a0)
                move.l  d1,220*2*19(a0)
                move.l  d1,220*2*20(a0)
                rts


music_vol:      .dc.w   255
effects_vol:    .dc.w   255
m_vol:          .dc.w   217*256
e_vol:          .dc.w   217*256
pause_opt:      .dc.w   0
pause_adds:     .dc.l   0
music_adds:     .dc.l   0
fx_adds:        .dc.l   0
arrow_adds:     .dc.l   0
red_adds:       .dc.l   0
store_channels: .dcb.l  66,0

DISPLAY_END_OF_PROGGY:

* Fade out screen
* Print congratulations etc...
* Wait for pad, then rts
                move.w  TIME_LEFT,d0           ; add final time bonus
                ext.w   d0
                mulu    #100,d0
                bsr     CONVERT_RAW_TO_BCD
                bsr     add_scoret

                move.w  LIVES,d7               ; add final lives bonus
                mulu    #10,d7                 ; add 5000 (10 * Lives)
.more:          move.w  #5000,d0               ; = 50,000 per life
                bsr     CONVERT_RAW_TO_BCD
                bsr     add_scoret
                dbf     d7,.more

                bsr     print_score

.set_last:      lea     score_asc,a0
                lea     life_bonus,a2
                move.b  (a0),(a2)
                move.b  1(a0),1(a2)
                move.b  2(a0),2(a2)
                move.b  3(a0),3(a2)
                move.b  4(a0),4(a2)
                move.b  5(a0),5(a2)
                move.b  6(a0),6(a2)
                move.b  7(a0),7(a2)

                clr.w   passwd_pos
                lea     passwd_asc,a0
                moveq   #7,d0
.clr:           clr.b   (a0)+
                dbf     d0,.clr

                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     clear_screen
                lea     GAME_COMPLETE_TEXT,a0
                bsr     print_string
                lea     intro_pal,a0
                bsr     set_palette16
;               moveq   #10,d0
;               bsr     wait_a_mo

.loop:          bsr     readpad
                move.l  joycur,d0
.fireb:         moveq   #FIRE_B,d1
                btst.l  d1,d0
                bne     .exit
                jsr     wait_int
                bra.s   .loop
.exit:
                lea     blank_pal,a0
                bsr     fade_2_pal
                move.w  #0,START_LEVEL

                rts

LEVEL_COMPLETED:
                tst.w   player_dead
                bne     GAME_LOOP

                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .next
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample

.next:          lea     TIME_LEFT,a0
                moveq   #0,d1
                move.b  (a0)+,d1
                mulu    #100,d1
                move.w  d1,d0
                moveq   #0,d1
                move.b  (a0),d1
                lsr.w   #4,d1
                mulu    #10,d1
                add.w   d1,d0
                moveq   #0,d1
                move.b  (a0)+,d1
                and.w   #$f,d1
                add.w   d1,d0
                move.w  d0,TIME_LEFT

                bsr     fill_logical_complete_box
                addq.w  #1,LIVES

                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     med_rez
                lea     WOW_TEXT,a0
                bsr     pro_print
                clr.w   bonus
                move.w  TIME_LEFT,d0
                bsr     set_bonus
                lea     TIME_BONUS_PRINT,a0
                bsr     pro_print
                lea     pro_pal,a0
                bsr     fade_2_pal
                bsr     do_bonus
                moveq   #2,d0
                bsr     wait_a_mo
                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     low_rez

                add.w   #1,mod_loop
                cmp.w   #3,mod_loop
                bne.s   .skip
                move.w  #0,mod_loop
                add.w   #1,curr_mod
                cmp.w   #12,curr_mod
                bne.s   .play
                move.w  #0,curr_mod
.play:          lea     mod_table,a1
                move.w  curr_mod,d0
                lsl.l   #2,d0
                add.l   d0,a1
                move.l  (a1),a0
                jsr     player+4                ;init new mod
.skip:
                bra     GAME_LOOP

wait_a_mo:
                move.w  time_loop_value,d1
                mulu    d0,d1
.wait:          bsr     wait_int
                move.w  d1,-(a7)
                bsr     readpad
                move.w  (a7)+,d1
                tst.l   joycur
                bne.s   .ex
                dbf     d1,.wait
.ex:            rts

do_bonus:
.loop:          bsr     wait_int
                bsr     wait_int
                bsr     wait_int
                bsr     del_bonus_txt
                move.w  TIME_LEFT,d0
                beq.s   .end
                subq.w  #1,d0
                move.w  d0,TIME_LEFT
                add.w   #1,bonus
.go:            bsr     set_bonus
.end:           lea     TIME_BONUS_PRINT,a0
                bsr     pro_print

                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp13_end-samp13,d0   ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #20,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp13,a0               ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1

                tst.w   TIME_LEFT
                beq.s   .ex
                bra.s   .loop
.ex:            move.w  bonus,d0
                ext.w   d0
                mulu    #100,d0
                bsr     CONVERT_RAW_TO_BCD
                bsr     add_scoret
                rts
add_scoret:     move.l  d1,add_score
                movem.l d0-a6,-(a7)
                move.l  #SCORE+4,a5
                move.l  #add_score+4,a6
                move    #4,ccr
                abcd    -(a6),-(a5)
                abcd    -(a6),-(a5)
                abcd    -(a6),-(a5)
                abcd    -(a6),-(a5)
                movem.l (a7)+,d0-a6
                rts
del_bonus_txt:  lea     pro_obs,a0
.srch:          cmp.l   #-1,(a0)+
                bne.s   .srch
                lea     -14*4(a0),a0
                bsr     wait_int
                bra     del_fnt_obs2

set_bonus:      lea     BONUS_TIME,a0
                moveq   #'0',d1
                ext.l   d0
                divu    #100,d0
                add.b   d1,d0
                move.b  d0,(a0)+
                clr.w   d0
                swap    d0
                divu    #10,d0
                add.b   d1,d0
                move.b  d0,(a0)+
                swap    d0
                add.b   d1,d0
                move.b  d0,(a0)+
                moveq   #0,d0
                move.w  bonus,d0
                lea     BONUS_SCORE,a0
                divu    #100,d0
                add.b   d1,d0
                move.b  d0,(a0)+
                clr.w   d0
                swap    d0
                divu    #10,d0
                add.b   d1,d0
                move.b  d0,(a0)+
                swap    d0
                add.b   d1,d0
                move.b  d0,(a0)+
                rts

INITIALISE_GAME_DATA:
                move.l  #"EGB!",box_stack
                move.w  START_LEVEL,LEVEL_NUMBER
                sf      GAME_ABORTED
                move.w  NO_OF_LIVES,LIVES
                clr.l   SCORE
                move.w  time_loop_value,TIME_FRAME
                rts
game_init:
.1:             cmp.w   #0,level_file
                bne     .2
                lea     level_file_data_1,a0
                lea     level_file_data,a1
                bra.s   .copy
.2:             cmp.w   #1,level_file
                bne     .3
                lea     level_file_data_2,a0
                lea     level_file_data,a1
                bra.s   .copy
.3:             cmp.w   #2,level_file
                bne     .copy
                lea     level_file_data_3,a0
                lea     level_file_data,a1
.copy:          move.w  #1999,d7
.loop:          move.l  (a0)+,(a1)+
                move.l  (a0)+,(a1)+
                move.l  (a0)+,(a1)+
                move.l  (a0)+,(a1)+
                move.l  (a0)+,(a1)+
                move.l  (a0)+,(a1)+
                move.l  (a0)+,(a1)+
                move.l  (a0)+,(a1)+
                dbf     d7,.loop
.next:          move.w  level_file_data+12,no_of_levels
                move.w  #1,final_levels
                move.w  no_of_levels,d0
                lea     level_space+16,a0
                move.l  a0,d1
.l_1:           add.l   d1,(a0)
                add.l   d1,4(a0)
                lea     32(a0),a0
                subq.w  #1,d0
                bne     .l_1
                rts

OCCASIONAL_STUFF:
                subq.w  #1,TIME_FRAME
                bpl     ocs_2

                move.w  time_loop_value,TIME_FRAME

                lea     TIME_LEFT+2,a0
                lea     BOLLOX,a1
                move.w  #1,(a1)+
                move.w  #4,CCR
                sbcd    -(a1),-(a0)
                sbcd    -(a1),-(a0)


play_secs20:    cmp.w   #$0020,TIME_LEFT
                bne.s   .10
                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp2_end-samp2,d0     ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #19,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp2,a0                ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1
.10:
play_secs10:    cmp.w   #$0010,TIME_LEFT
                bne.s   .05
                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp3_end-samp3,d0     ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #19,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp3,a0                ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1
.05:
play_secs08:    cmp.w   #$0008,TIME_LEFT
                bgt.s   .00
                cmp.w   #$0001,TIME_LEFT
                blt.s   .00
                move.w  beepsamp,d0
                cmp.w   #0,d0
                beq.s   .next
                move.w  #1,d1
                jsr     player+$10              ;Turn off beep sample
                clr.w   beepsamp
.next:
                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp13_end-samp13,d0   ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #19,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp13,a0               ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                move.w  d0,beepsamp
                movem.l (a7)+,d0-d4/a0-a1
.00:
                cmp.w   #0,TIME_LEFT
                beq     NO_TIME_LEFT
                bsr     print_time
ocs_2:
                rts
print_time:     movem.l d0-a6,-(a7)
                lea     time_asc,a1
                move.w  TIME_LEFT,d0
                rol.w   #8,d0
                move.b  d0,d1
                and.w   #$f,d1
                add.w   #'0',d1
                move.b  d1,(a1)+
                rol.w   #4,d0
                move.b  d0,d1
                and.w   #$f,d1
                add.w   #'0',d1
                move.b  d1,(a1)+
                rol.w   #4,d0
                move.b  d0,d1
                and.w   #$f,d1
                add.w   #'0',d1
                move.b  d1,(a1)+
                lea     time_text,a0
                bsr     print_string
                movem.l (a7)+,d0-a6
                rts

NO_TIME_LEFT:
                tst.b   cheat
                beq.s   .no_cheat
                move.w  #1,TIME_LEFT
                rts
.no_cheat:
.secs00:
                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .next
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
.next:
                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp5_end-samp5,d0     ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #20,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp5,a0                ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1

                st      player_dead
                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     med_rez
                lea     TIMEOUT_TEXT,a0
                bsr     pro_print
                lea     pro_pal,a0
                bsr     fade_2_pal
                moveq   #2,d0
                bsr     wait_a_mo
                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     low_rez

                rts

draw_level:
                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     CONVERT_TIME_TO_ASCII
                bsr     display_level_name


clear_hole_table:
                lea     hole_table,a0
                move.w  #35,d0
.loop:          move.l  #"FREE",(a0)+
                dbf     d0,.loop

clear_box_stack:
                lea     box_stack,a0
                move.l  #"EGB!",(a0)
                clr.l   set_address
                clr.l   clear_address
                clr.l   flip_address
                bsr     clear_box_table

clear_nasty_space:
                lea     nasty_space,a0
                moveq   #0,d7
                move.w  #4,d6           ;number of nasties
.loop:          move.l  d7,(a0)
                move.l  d7,4(a0)
                add.l   #nasty_rs_length,a0
                dbf     d6,.loop

clear_rest:

                move.w  #0,box_height
                moveq.l #-1,d0
                move.w  d0,no_of_boxes

                lea     level_space+16,a0
                move.w  LEVEL_NUMBER,d0        ;Each record is 32 bytes
                lsl.w   #5,d0
                move.l  (a0,d0.w),a0           ;PTR to Level Coords

                lea     box_table,a1
                sf      hidden
.loop:
                move.w  (a0)+,d5
                bmi     .exit

                move.w  #8,line_colour
                move.w  18(a0),d6
                beq     .skip
                move.w  #4,line_colour
                st      hidden

.skip:          bsr     count_pixels
                move.w  d7,(a1)+
                move.w  d5,(a1)+

                addq.w  #1,no_of_boxes

                move.w  d5,d0
                move.w  (a0)+,d1
                move.w  (a0)+,d2
                move.w  d2,(a1)+
                move.w  d1,(a1)+
                move.w  (a0)+,d3
                movem.l d0-d5/a0-a1,-(a7)
                moveq   #0,d4
                bsr     draw_horiz
                movem.l (a7)+,d0-d5/a0-a1
                move.w  d2,d4
                move.w  d3,d5
                move.w  (a0)+,d2
                move.w  (a0)+,d3
                move.w  d3,(a1)+
                movem.l d0-d5/a0-a1,-(a7)
                bsr     draw_vert
                movem.l (a7)+,d0-d5/a0-a1
                move.w  d2,d0
                move.w  d3,d1
                move.w  (a0)+,d2
                move.w  (a0)+,d3
                movem.l d0-d5/a0-a1,-(a7)
                moveq   #0,d4
                bsr     draw_horiz
                movem.l (a7)+,d0-d5/a0-a1
                move.w  d4,d0
                move.w  d5,d1
                movem.l d0-d5/a0-a1,-(a7)
                bsr     draw_vert
                movem.l (a7)+,d0-d5/a0-a1
                move.l  (a0)+,(a1)+
                addq.l  #2,a0
                clr.w   (a1)+
                bra     .loop
.exit:          move.w  no_of_boxes,no_of_boxes_left

                lea     level_space+16,a0
                move.w  LEVEL_NUMBER,d0        ;Each block is 32 bytes
                lsl.w   #5,d0
                move.w  10(a0,d0.w),d1         ;No. of Nasties
                move.l  4(a0,d0.w),a1          ;PTR to Nasty data

                move.w  -4(a1),me_x
                move.w  -2(a1),me_y

                lea     nasty_space,a2
                subq.w  #1,d1
                move.w  d1,no_of_nasties
.l_1:
                move.w  (a1)+,nasty_type(a2)
                move.w  (a1)+,nasty_x(a2)
                move.w  (a1)+,nasty_y(a2)
                clr.w   nasty_dir(a2)

                addq.l  #nasty_rs_length,a2
                dbf     d1,.l_1

                lea     level_space+16,a0
                move.w  LEVEL_NUMBER,d0
                lsl.w   #5,d0
                add.w   d0,a0
                add.w   #12,a0

                lea     level_name_text,a1
                move.w  #19,d0
.l_2:
                move.b  (a0)+,(a1)+
                dbf     d0,.l_2

                bsr     CONVERT_TIME_TO_BCD

                bsr     print_lives
                lea     LINE_TEXT,a0
                bsr     print_string
                bsr     print_time
                bsr     print_score
                moveq   #0,d0
                moveq   #8,d1
                move.w  #319,d2
                moveq   #0,d4
                move.w  #7,line_colour
                bsr     draw_horiz
                move.w  #$ffff,colour_4
                tst.b   hidden
                beq.s   .no_hidden
                move.w  #$ffff,colour_12
.no_hidden:     lea     game_pal,a0
                bsr     fade_2_pal
                bsr     draw_player
                bsr     draw_nasties
                clr.w   colour_4
                move.w  #$e000,colour_12
                lea     game_pal,a0
                bsr     fade_2_pal
                rts
print_lives:    lea     lives_asc,a0
                moveq   #0,d0
                move.w  LIVES,d0
                divu    #10,d0
                add.b   #'0',d0
                move.b  d0,(a0)+
                swap    d0
                add.b   #'0',d0
                move.b  d0,(a0)+
                rts

CONVERT_TIME_TO_ASCII:
                lea     level_space+16,a0
                move.w  LEVEL_NUMBER,d0        ;Each record is 32 bytes
                lsl.w   #5,d0
                move.w  8(a0,d0.w),d0          ;Timer Value
                ext.l   d0
                divu    time_loop_value,d0

                lea     LEVEL_TIME_TEXT,a0
                bra     CONVERT_RAW_NUM_TO_ASCII

CONVERT_TIME_TO_BCD:
                lea     level_space+16,a0
                move.w  LEVEL_NUMBER,d0        ;Each record is 32 bytes
                lsl.w   #5,d0
                move.w  8(a0,d0.w),d0          ;Timer Value
                ext.l   d0
                divu    time_loop_value,d0
                bsr     CONVERT_RAW_TO_BCD
                move.w  d1,TIME_LEFT
                rts

CONVERT_RAW_TO_BCD:
                moveq   #0,d1
                move.w  #16,d3
                move.w  #10000,d2
                bsr     sub_num_2
                lsl.l   d3,d4
                or.l    d4,d1
                subq.w  #4,d3

                move.w  #1000,d2
                bsr     sub_num_2
                lsl.l   d3,d4
                or.l    d4,d1
                subq.w  #4,d3

                move.w  #100,d2
                bsr     sub_num_2
                lsl.l   d3,d4
                or.l    d4,d1
                subq.w  #4,d3

                move.w  #10,d2
                bsr     sub_num_2
                lsl.l   d3,d4
                or.l    d4,d1
                subq.w  #4,d3

                move.w  #1,d2
                bsr     sub_num_2
                or.l    d4,d1

                rts
sub_num_2:
                moveq   #0,d4
.l_1:
                addq.w  #1,d4
                sub.w   d2,d0
                bpl     .l_1
                subq.w  #1,d4
                add.w   d2,d0
                rts

display_level_name:

                bsr     med_rez

                move.w  LEVEL_NUMBER,d0
                lsl.w   #5,d0
                lea     level_space+16,a0
                add.w   d0,a0
                lea     LEVEL_NAME,a1
                move.l  12(a0),(a1)+
                move.l  16(a0),(a1)+
                move.l  20(a0),(a1)+
                move.l  24(a0),(a1)+
                move.l  28(a0),(a1)+
                move.l  4(a0),a1               ;PTR to Nasty Data
                lea     -12(a1),a1             ;Now Points to the Password

                eor.l   #$895623ff,(a1)
                eor.l   #$14567821,4(a1)

                pea     (a1)
                tst.w   (a1)
                beq     .skip_password

                lea     PASSWORD_TEXT_STRING,a0
                move.b  (a1)+,(a0)
                move.b  (a1)+,1(a0)
                move.b  (a1)+,2(a0)
                move.b  (a1)+,3(a0)
                move.b  (a1)+,4(a0)
                move.b  (a1)+,5(a0)
                move.b  (a1)+,6(a0)
                move.b  (a1)+,7(a0)

                lea     PASSWORD_TEXT,a0
                bsr     pro_print

.skip_password:
                move.l  (a7)+,a1
                eor.l   #$895623ff,(a1)
                eor.l   #$14567821,4(a1)

                move.w  LEVEL_NUMBER,d0
                lea     LEVEL_NUMBER_TEXT,a0
                bsr     CONVERT_RAW_NUM_TO_ASCII

                lea     LEVEL_TEXT,a0
                bsr     pro_print

                lea     pro_pal,a0
                bsr     fade_2_pal
                moveq   #5,d0
                bsr     wait_a_mo
                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     low_rez
                rts

display_game_over:
                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .next
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
.next:
                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     med_rez
                lea     GAME_OVER_TEXT,a0
                bsr     pro_print
                lea     pro_pal,a0
                bsr     fade_2_pal
                moveq   #2,d0
                bsr     wait_a_mo
                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     low_rez
                rts

DISPLAY_GAME_ABORTED:
                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     med_rez
                lea     GAME_ABORTED_TEXT,a0
                bsr     pro_print
                lea     pro_pal,a0
                bsr     fade_2_pal
                st      GAME_ABORTED

                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .next
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
.next:
                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp6_end-samp6,d0     ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #20,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp6,a0                ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1

                moveq   #2,d0
                bsr     wait_a_mo
                lea     blank_pal,a0
                bsr     fade_2_pal
                bsr     low_rez
                rts

CONVERT_RAW2ASCII_2:
                move.w  #10000,d1
                bsr     sub_num
                move.w  #1000,d1
                bsr     sub_num
                move.w  #100,d1
                bsr     sub_num
                move.w  #10,d1
                bsr     sub_num
                move.w  #1,d1
                bsr     sub_num
                rts

CONVERT_RAW_NUM_TO_ASCII:
                move.w  #100,d1
                bsr     sub_num
                move.w  #10,d1
                bsr     sub_num
                move.w  #1,d1
                bsr     sub_num
                rts
sub_num:
                moveq   #0,d2
.l_1:
                addq.w  #1,d2
                sub.w   d1,d0
                bpl     .l_1
                subq.w  #1,d2
                add.w   d1,d0
                add.w   #"0",d2
                move.b  d2,(a0)+
                rts
count_pixels:
                move.w  -2(a0),d0       ;first  x coord
                move.w  2(a0),d6        ;second x coord
                sub.w   d0,d6           ;get difference
                add.w   d6,d6           ;double it (2 sides)
                move.w  (a0),d1         ;first  y coord
                move.w  8(a0),d7        ;second y coord
                sub.w   d1,d7           ;get difference
                add.w   d7,d7           ;double it
                add.w   d6,d7           ;add x to y
                rts

draw_sprite:    movem.l d0-d2/a0-a1,-(a7)
                move.l  a1,d2
                and.l   #$ff,(a0)
                and.b   #$f8,d2
                lsl.l   #8,d2
                or.l    d2,(a0)
                and.w   #$c007,6(a0)
                lsl.l   #3,d1
                and.w   #$3ff8,d1
                or.w    d1,6(a0)
                and.w   #$f000,14(a0)
                and.w   #$fff,d0
                or.w    d0,14(a0)
                movem.l (a7)+,d0-d2/a0-a1
                rts

is_player_dead:
                tst.w   player_dead
                beq.s   .exit

                cmp.w   #0,TIME_LEFT
                beq.s   .skip

                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .next
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
.next:
                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp14_end-samp14,d0   ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #18,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp14,a0               ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1
.skip:
                subq.w  #1,LIVES
                bpl.s   .ok
                st      totally_dead_dude
                bra.s   .exit
.ok:            st      skip_level_flag
*               clr.w   player_dead
.exit:          rts

next_level_yet:
                tst.w   no_of_boxes_left
                bpl     .exit

                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp15_end-samp15,d0   ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #18,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp15,a0               ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1

                move.w  LEVEL_NUMBER,d0
                addq.w  #5,d0
                subq.w  #4,d0
                move.w  d0,LEVEL_NUMBER
                move.l  d0,-(a7)
.update_restart_passwd:
                move.w  d0,d1
                lsl.w   #5,d1
                lea     level_space+16,a0
                lea     (a0,d1.w),a0
                lea     12(a0),a3
                move.l  4(a0),a1               ;PTR to Nasty Data
                lea     -12(a1),a1             ;Now Points to the Password
                move.l  (a1)+,d6
                eor.l   #$895623ff,d6
                move.l  (a1)+,d7
                eor.l   #$14567821,d7
                lea     passwd_asc,a6
                move.l  d6,(a6)
                move.l  d7,4(a6)
                move.b  #0,8(a6)
                cmp.b   #0,(a6)+
                bne     .2
                move.w  #0,passwd_pos
                bra     .9
.2:             cmp.b   #0,(a6)+
                bne     .3
                move.w  #0,passwd_pos
                bra     .9
.3:             cmp.b   #0,(a6)+
                bne     .4
                move.w  #1,passwd_pos
                bra     .9
.4:             cmp.b   #0,(a6)+
                bne     .5
                move.w  #2,passwd_pos
                bra     .9
.5:             cmp.b   #0,(a6)+
                bne     .6
                move.w  #3,passwd_pos
                bra     .9
.6:             cmp.b   #0,(a6)+
                bne     .7
                move.w  #4,passwd_pos
                bra     .9
.7:             cmp.b   #0,(a6)+
                bne     .8
                move.w  #5,passwd_pos
                bra     .9
.8:             cmp.b   #0,(a6)+
                bne     .9a
                move.w  #6,passwd_pos
                bra     .9
.9a:            move.w  #7,passwd_pos
.9:             move.l  (a7)+,d0
                move.w  d0,START_LEVEL
                cmp.w   no_of_levels,d0
                st      skip_level_flag
                blt     .exit
                st      HA_HA_NO_MORE_PLAY_TODAY
                st      totally_dead_dude
                sf      skip_level_flag
.exit:          rts


draw_horiz:     cmp.w   d0,d2
                bgt.s   .x_ok
                beq     .ex
                exg     d0,d2
.x_ok:
.loop:          movem.l d0-d2,-(a7)
                lea     screen_32k,a0
                sub.w   d0,d2
                move.w  line_colour,d7
                move.w  d7,d6
                lsl.w   #4,d7
                or.w    d6,d7
                clr.b   d6
                lsr.w   #1,d0
                bcc.s   .left_ok
                move.b  d7,d6
                and.b   #$f,d6
                addq.w  #1,d0
                subq.w  #1,d2
.left_ok:       lea     (a0,d0.w),a0
                move.w  d1,d3
                lsl.w   #7,d1
                lsl.w   #5,d3
                add.w   d3,d1
                lea     (a0,d1.w),a0
                or.b    d6,-1(a0)
                clr.b   d6
                lsr.w   #1,d2
                bcc.s   .right_ok
                move.b  d7,d6
                and.b   #$f0,d6
.right_ok:      subq.w  #1,d2
.draw:          or.b    d7,(a0)+
                dbf     d2,.draw
                or.b    d6,(a0)
                movem.l (a7)+,d0-d2
                addq.w  #1,d1
                dbf     d4,.loop
.ex:
                rts

draw_vert:      cmp.w   d1,d3
                bgt.s   .y_ok
                beq     .ex
                exg     d1,d3
.y_ok:          lea     screen_32k,a0
                move.w  line_colour,d7
                lsr.w   #1,d0
                bcs.s   .pixel_odd
                lsl.w   #4,d7
.pixel_odd:     lea     (a0,d0.w),a0
                sub.w   d1,d3
                move.w  d1,d0
                lsl.w   #7,d1
                lsl.w   #5,d0
                add.w   d0,d1
                lea     (a0,d1.w),a0
.draw:          or.b    d7,(a0)
                lea     160(a0),a0
                dbf     d3,.draw
.ex:
                rts

move_sprites:   tst.b   K_F
                bne     .fire
.fire_return:   tst.b   move_priority
                bne.s   .chk_lr
.chk_ud:        tst.b   K_U
                bne     .move_up
.up_return:     tst.b   K_D
                bne     .move_down
.down_return:
                tst.b   move_priority
                bne.s   .test_cheat

.chk_lr:        tst.b   K_L
                bne     .move_left
.left_return:   tst.b   K_R
                bne     .move_right
.right_return:
                tst.b   move_priority
                bne.s   .chk_ud
.test_cheat:
                tst.b   cheat
                beq.s   .exit
                tst.b   K_3
                bne     .skip_level_f
                tst.b   K_1
                bne     .skip_level_b
.exit:          rts


.skip_level_f:  move.w  LEVEL_NUMBER,d0
                addq.w  #1,d0
                cmp.w   no_of_levels,d0
                blt     .sf
                moveq   #0,d0
.sf:            move.w  d0,LEVEL_NUMBER
                st      skip_level_flag
                bra     .exit

.skip_level_b:  move.w  LEVEL_NUMBER,d0
                subq.w  #1,d0
                bpl     .sb
                move.w  no_of_levels,d0
                subq.w  #1,d0
.sb:            move.w  d0,LEVEL_NUMBER
                st      skip_level_flag
                bra     .exit

.move_up:       tst.w   valid_up
                bne     .play_up_sound
                cmp.w   #0,me_dir
                bne.s   .ur
                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .ur
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
                clr.w   movesamp
.ur:            bra     .up_return

.play_up_sound: cmp.w   #0,me_dir
                beq.s   .u
                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .uc
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
.uc:            movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp16_end-samp16,d0   ;sample length
                move.l  d0,d1                   ;loop length
                moveq   #15,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp16,a0               ;start address
                move.l  a0,a1                   ;loop start address
                jsr     player+$c
                move.w  d0,movesamp
                movem.l (a7)+,d0-d4/a0-a1
.u:
                move.w  #1,move_occur
                move.w  #0,me_dir
                sub.w   #1,me_y
                st      move_priority
                bra     .exit

.move_down:     tst.w   valid_down
                bne     .play_dn_sound
                cmp.w   #1,me_dir
                bne.s   .dr
                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .dr
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
                clr.w   movesamp
.dr:            bra     .down_return

.play_dn_sound: cmp.w   #1,me_dir
                beq.s   .d
                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .dc
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
.dc:            movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp17_end-samp17,d0   ;sample length
                move.l  d0,d1                   ;looping length
                moveq   #15,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp17,a0               ;start address
                move.l  a0,a1                   ;loop start address
                jsr     player+$c
                move.w  d0,movesamp
                movem.l (a7)+,d0-d4/a0-a1
.d:
                move.w  #1,move_occur
                move.w  #1,me_dir
                add.w   #1,me_y
                st      move_priority
                bra     .exit

.move_right:    tst.w   valid_right
                bne     .play_ri_sound
                cmp.w   #2,me_dir
                bne.s   .rr
                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .rr
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
                clr.w   movesamp
.rr:            bra     .right_return

.play_ri_sound: cmp.w   #2,me_dir
                beq.s   .r
                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .rc
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
.rc:            movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp18_end-samp18,d0   ;sample length
                move.l  d0,d1                   ;looping length
                moveq   #15,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp18,a0               ;start address
                move.l  a0,a1                   ;loop start address
                jsr     player+$c
                move.w  d0,movesamp
                movem.l (a7)+,d0-d4/a0-a1
.r:
                move.w  #1,move_occur
                move.w  #2,me_dir
                add.w   #1,me_x
                sf      move_priority
                bra     .exit

.move_left:     tst.w   valid_left
                bne     .play_le_sound
                cmp.w   #3,me_dir
                bne.s   .lr
                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .lr
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
                clr.w   movesamp
.lr:            bra     .left_return

.play_le_sound: cmp.w   #3,me_dir
                beq.s   .l1
                move.w  movesamp,d0
                cmp.w   #0,d0
                beq.s   .lc
                move.w  #1,d1
                jsr     player+$10              ;Turn off move sample
.lc:            movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp19_end-samp19,d0   ;sample length
                move.l  d0,d1                   ;looping length
                moveq   #15,d2                  ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp19,a0                ;start address
                move.l  a0,a1                   ;loop start address
                jsr     player+$c
                move.w  d0,movesamp
                movem.l (a7)+,d0-d4/a0-a1
.l1:
                move.w  #1,move_occur
                move.w  #3,me_dir
                sub.w   #1,me_x
                sf      move_priority
                bra     .exit

.fire:          lea     hole_table,a0
                cmp.l   #"FREE",(a0)
                beq     .fire_ok
                cmp.l   #"FREE",12*4(a0)
                beq     .fire_ok
                cmp.l   #"FREE",24*4(a0)
                bne     .fire_return
.fire_ok:
                tst.w   erase_pixel_flag
                bne     .fire_return
                move.w  #1,erase_pixel_flag
                bra     .fire_return

*******************************
* Call sprite rout for player *
*******************************
draw_player:    move.w  me_x,d0
                move.w  me_y,d1
                move.w  me_dir,d2
                add.w   #scr_x-7,d0
                add.w   d1,d1
                add.w   a_vdb,d1
                sub.w   #14,d1
                lsl.w   #7,d2
                lea     sprites,a1
                lea     (a1,d2.w),a1
                move.l  player_obj,a0
                bra     draw_sprite

********************************
* Call sprite rout for nasties *
********************************
draw_nasties:
                lea     nasty_space,a2
                lea     nasty_obj1,a3
                move.w  no_of_nasties,d7
.l_1:
                move.w  nasty_x(a2),d0
                move.w  nasty_y(a2),d1
                move.w  nasty_type(a2),d2
                add.w   d2,d2
                add.w   d2,d2
                add.w   nasty_dir(a2),d2
                addq.w  #4,d2

                add.w   #scr_x-7,d0
                add.w   d1,d1
                add.w   a_vdb,d1
                sub.w   #14,d1
                lsl.w   #7,d2
                lea     sprites,a1
                lea     (a1,d2.w),a1
                move.l  (a3)+,a0
                bsr     draw_sprite

                addq.l  #nasty_rs_length,a2
                dbf     d7,.l_1
                rts


test_me_direction:
                move.w  me_x,check_x
                move.w  me_y,check_y

                moveq.l #0,d2
                move.l  d2,valid_up
                move.l  d2,valid_left
                move.w  check_x,d0
                move.w  check_y,d1

                lea     screen_32k,a0
                move.w  d1,d2
                lsl.w   #5,d1
                lsl.w   #7,d2
                add.w   d2,d1
                lea     (a0,d1.w),a0
                move.w  #$f0,d2
                lsr.w   #1,d0
                bcc.s   .pos_ok
                lsr.w   #4,d2
.pos_ok:        lea     (a0,d0.w),a0
                move.l  a0,erase_address
                move.b  d2,erase_d0

check_under:    move.b  (a0),d1
                and.w   d2,d1
                btst    #4,d2
                beq.s   .ok
                lsr.w   #4,d1
.ok:            cmp.b   #12,d1
                beq     .doit
                cmp.b   #8,d1
                beq     .doit
                cmp.b   #4,d1
                beq     .doit
                bra     check_up
.doit:          moveq   #9,d1
                not.b   d2
                and.b   d2,(a0)
                btst    #0,d2
                beq.s   .ok2
                lsl.w   #4,d1
.ok2:           or.b    d1,(a0)
                not.b   d2
                movem.l d0-a6,-(a7)
                bsr     update_box_data
                moveq   #1,d1
                bsr     add_scoret
                bsr     print_score
                movem.l (a7)+,d0-a6

********************************
* Check whether we can move up *
********************************
check_up:       moveq.l #0,d1
                move.b  -160(a0),d1
                and.w   d2,d1
                btst    #4,d2
                beq.s   .ok
                lsr.w   #4,d1
.ok:            cmp.b   #4,d1
                bne     .1
                move.b  #8,d1
.1:             cmp.b   #8,d1
                blt     check_down
                move.w  #1,valid_up
check_down:     move.b  160(a0),d1
                and.w   d2,d1
                btst    #4,d2
                beq.s   .ok
                lsr.w   #4,d1
.ok:            cmp.b   #4,d1
                bne     .1
                move.b  #8,d1
.1:             cmp.b   #8,d1
                blt     check_left
                move.w  #1,valid_down
check_left:     not.w   d2
                move.b  (a0),d1
                btst    #0,d2
                beq.s   .ok
                move.b  -1(a0),d1
.ok:            and.w   d2,d1
                btst    #4,d2
                beq.s   .ok2
                lsr.w   #4,d1
.ok2:           cmp.b   #4,d1
                bne     .1
                move.b  #8,d1
.1:             cmp.b   #8,d1
                blt     check_right
                move.w  #1,valid_left
check_right:    move.b  (a0),d1
                btst    #0,d2
                bne.s   .ok
                move.b  1(a0),d1
.ok:            and.w   d2,d1
                btst    #4,d2
                beq.s   .ok2
                lsr.w   #4,d1
.ok2:           cmp.b   #4,d1
                bne     .1
                move.b  #8,d1
.1:             cmp.b   #8,d1
                blt     check_end
                move.w  #1,valid_right
check_end:      rts

**********************************
* Clear pixels when fire pressed *
**********************************
erase_pixels:
                tst.w   move_occur
                beq     .exit

                clr.w   move_occur
                tst.w   erase_pixel_flag
                beq     .exit

                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp7_end-samp7,d0     ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #150,d2                 ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp7,a0                ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1

                lea     hole_table,a5
.loop:          cmp.l   #"EGB!",(a5)    ;end of table
                beq     .exit
                cmp.l   #"FREE",(a5)    ;free entry
                bne     .next

                move.l  erase_address,a0
                move.b  erase_d0,d0

                move.b  #$99,d1
                and.b   d0,d1
                beq.s   .exit
                move.b  d1,(a5)
                move.l  a0,4(a5)        ;store address
                move.l  #500,8(a5)      ;life

                not.b   d0
                and.b   d0,(a0)

                addq.w  #1,erase_pixel_flag
                cmp.w   #5,erase_pixel_flag
                blt     .exit
                clr.w   erase_pixel_flag
                bra     .exit

.next:          add.l   #12,a5
                bra     .loop
.exit:          rts

***********************************
* Update Hole table               *
***********************************
update_hole_table:
                lea     hole_table,a5
.loop:          cmp.l   #"EGB!",(a5)    ;end of table
                beq     .exit

                cmp.l   #"FREE",(a5)
                beq     .next

                move.l  8(a5),d7        ;check life
                subq.l  #1,d7
                beq     .clear

                move.l  d7,8(a5)
                bra     .next

.clear:
                move.l  4(a5),a0
                move.b  (a5),d1
                or.b    d1,(a0)
                move.l  #"FREE",(a5)            ;blank table entry
                move.l  #"FREE",4(a5)
                move.l  #"FREE",8(a5)

.next:          add.l   #12,a5
                bra     .loop
.exit:          rts




***********************************
* Find out where nasty can go now *
***********************************
test_nasty_direction:
                moveq.l #0,d2
                move.l  d2,valid_up
                move.l  d2,valid_left
                move.w  check_x,d0
                move.w  check_y,d1

                lea     screen_32k,a0
                move.w  d1,d2
                lsl.w   #5,d1
                lsl.w   #7,d2
                add.w   d2,d1
                lea     (a0,d1.w),a0
                move.w  #$f0,d2
                lsr.w   #1,d0
                bcc.s   .pos_ok
                lsr.w   #4,d2
.pos_ok:        lea     (a0,d0.w),a0
                cmp.w   #1,nasty_dir(a2)
                beq     pre_check_down

check_up1:      moveq   #0,d1
                move.b  -160(a0),d1
                and.w   d2,d1
                btst    #4,d2
                beq.s   .ok
                lsr.w   #4,d1
.ok:            cmp.b   #4,d1
                bne     .1
                move.b  #8,d1
.1:             cmp.b   #8,d1
                blt     pre_check_down
                move.w  #1,valid_up
                addq.w  #1,d6
pre_check_down:
                tst.w   nasty_dir(a2)
                beq     pre_check_left
check_down1:    move.b  160(a0),d1
                and.w   d2,d1
                btst    #4,d2
                beq.s   .ok
                lsr.w   #4,d1
.ok:            cmp.b   #4,d1
                bne     .1
                move.b  #8,d1
.1:             cmp.b   #8,d1
                blt     pre_check_left
                move.w  #1,valid_down
                addq.w  #1,d6
pre_check_left:
                cmp.w   #2,nasty_dir(a2)
                beq     pre_check_right
check_left1:    not.w   d2
                move.b  (a0),d1
                btst    #0,d2
                beq.s   .ok
                move.b  -1(a0),d1
.ok:            and.w   d2,d1
                btst    #4,d2
                beq.s   .ok2
                lsr.w   #4,d1
.ok2:           cmp.b   #4,d1
                bne     .1
                move.b  #8,d1
.1:             cmp.b   #8,d1
                blt     pre_check_right
                move.w  #1,valid_left
                addq.w  #1,d6
pre_check_right:
                cmp.w   #3,nasty_dir(a2)
                beq     check_end1
check_right1:   move.b  (a0),d1
                btst    #0,d2
                bne.s   .ok
                move.b  1(a0),d1
.ok:            and.w   d2,d1
                btst    #4,d2
                beq.s   .ok2
                lsr.w   #4,d1
.ok2:           cmp.b   #4,d1
                bne     .1
                move.b  #8,d1
.1:             cmp.b   #8,d1
                blt     check_end1
                move.w  #1,valid_right
                addq.w  #1,d6
check_end1:     rts


****************************************************
* keep count of pixels left until box needs filled *
****************************************************
update_box_data:
                move.w  me_y,d7
                move.w  me_x,d5
                move.w  no_of_boxes,d6
                lea     box_table,a0
.loop:          cmp.w   6(a0),d7        ;check y1
                blt     .next
                cmp.w   8(a0),d7        ;check y2
                bgt     .next
                cmp.w   2(a0),d5        ;check x1
                blt     .next
                cmp.w   4(a0),d5        ;check x2
                bgt     .next
.found:         subq.w  #1,(a0)         ;reduce no. of pixels left
.next:          lea     16(a0),a0
                dbf     d6,.loop
                rts

check_for_complete_box:
                move.w  no_of_boxes,d6
                lea     box_table,a0
                lea     box_stack,a1
.l_1:
                cmp.l   #"EGB!",(a1)
                beq     .l_2
                tst.w   (a1)
                bmi     .l_2
                lea     10(a1),a1
                bra     .l_1
.l_2:
.loop:          tst.w   (a0)
                bne     .next

                subq.w  #1,no_of_boxes_left

                move.l  10(a0),add_score       ;update score
                move.l  #SCORE+4,a5
                move.l  #add_score+4,a6
                move    #4,ccr
                abcd    -(a6),-(a5)
                abcd    -(a6),-(a5)
                abcd    -(a6),-(a5)
                abcd    -(a6),-(a5)

                movem.l d0-d4/a0-a1,-(a7)
                move.l  #samp8_end-samp8,d0     ;sample length
                move.l  #0,d1                   ;looping length
                moveq   #160,d2                 ;priority
                moveq   #0,d3                   ;default volume
                moveq   #0,d4                   ;default frequency
                lea     samp8,a0                ;start address
                move.l  #0,a1                   ;loop start address
                jsr     player+$c
                movem.l (a7)+,d0-d4/a0-a1

                bsr     print_score

                move.w  2(a0),d0               ;x1 coord
                move.w  6(a0),d1               ;y1 coord
                move.w  4(a0),d2               ;x2 coord
                move.w  8(a0),d4               ;y2 coord
                sub.w   d1,d4                  ;find height of box
                addq.w  #1,d0                  ;add 1 to x pos
                addq.w  #1,d1                  ;add 1 to y pos
                subq.w  #1,d2                  ;sub 1 from end x pos
                subq.w  #2,d4                  ;sub 2 from height

                move.w  #1,(a1)+               ;Flag for count
                move.w  d0,(a1)+               ;        X1
                move.w  d1,(a1)+               ;        Y1
                move.w  d2,(a1)+               ;        X2
                move.w  d4,(a1)+               ;        Height

                move.w  #-1,(a0)               ;box has been filled

.next:          lea     16(a0),a0
                dbf     d6,.loop
                move.l  #"EGB!",(a1)           ;last box written
                rts

print_score:    movem.l d0-a6,-(a7)
                move.l  SCORE,d1
                lea     score_asc,a1
                moveq   #7,d0
.lp:            rol.l   #4,d1
                move.b  d1,d2
                and.w   #$f,d2
                add.b   #'0',d2
                move.b  d2,(a1)+
                dbf     d0,.lp
                lea     score_text,a0
                bsr     print_string
                movem.l (a7)+,d0-a6
                rts
clear_box_table:
                lea     box_table,a0
                move.w  #39,d6
                moveq.l #0,d7
.loop:          move.l  d7,(a0)
                move.l  d7,4(a0)
                move.l  d7,8(a0)
                move.l  d7,12(a0)
.next:          lea     16(a0),a0
                dbf     d6,.loop
                rts

*************************************
* draw filled box on logical screen *
*************************************
fill_logical_complete_box:
                move.w  #1,line_colour
                lea     box_stack,a0
.l_1:
                cmp.l   #"EGB!",(a0)
                beq     .exit
                tst.w   (a0)
                bmi     .next

                move.w  2(a0),d0
                move.w  4(a0),d1
                move.w  6(a0),d2
                move.w  8(a0),d4
                move.w  d4,box_height

                pea     (a0)
                addq.w  #1,d2
                bsr     draw_horiz
                move.l  (a7)+,a0

                subq.w  #1,(a0)
.next:          lea     10(a0),a0
                bra     .l_1
.exit:          rts

*******************************************************************
* Move the nasties around...                                      *
* Nasties will move in a given direction and only turn when they  *
* hit either a crossroads or a dead end.  If there are only       *
* two options for direction (the current one, and a new one), the *
* new one must be taken.                                          *
* If there is more than two options, then the beastie will take   *
* the option which it thinks will take it closer to you.          *
*******************************************************************
move_nasties:
                lea     nasty_space,a2
                move.w  no_of_nasties,d7
mn_1:
                move.l  #0,xline_flag
                move.w  nasty_x(a2),d2              ;   X coord
                move.w  nasty_y(a2),d3              ;   Y coord
                sub.w   me_x,d2
                sub.w   me_y,d3
.check_x:       tst.w   d2
                bpl.s   .check_dist_x
                neg.w   d2
.check_dist_x:
                btst    #1,me_dir+1
                beq     .d_x

                sub.w   #10,d2
                bpl     .next
                cmp.w   #-10,d2
                beq.s   .check_y
                move.w  #1,xline_flag
                bra     .check_y
.d_x:
                sub.w   #5,d2
                bpl.s   .next
                cmp.w   #-5,d2
                beq.s   .check_y
                move.w  #1,xline_flag
.check_y:       tst.w   d3
                bpl.s   .check_dist_y
                neg.w   d3
.check_dist_y:
                btst    #1,me_dir+1
                bne     .d_y

                sub.w   #10,d3
                bpl.s   .next
                cmp.w   #-10,d3
                beq.s   .final_check
                move.w  #1,yline_flag
                bra     .final_check
.d_y:
                sub.w   #5,d3
                bpl.s   .next
                cmp.w   #-5,d3
                beq.s   .final_check
                move.w  #1,yline_flag
.final_check:
                tst.w   xline_flag             ;is nasty on same x line
                beq.s   .dead                  ;
                tst.w   yline_flag             ;is nasty on same y line
                beq.s   .dead                  ;
                bra.s   .next                  ;just escaped!!!
.dead:                                         ;
                tst.b   cheat                  ;
                bne.s   .next                  ;
                move.w  #1,player_dead         ;
.next:                                         ;
                move.w  nasty_x(a2),check_x    ;        X coord
                move.w  nasty_y(a2),check_y    ;        Y coord
                moveq   #0,d6                  ;
                bsr     test_nasty_direction   ;
                                               ;
                subq.w  #1,d6                  ;
                beq     mn_7                   ;        Corner
                bmi     mn_14                  ;        Dead End
                                               ;
                moveq   #0,d6                  ;
                moveq   #0,d5                  ;
                                               ;
                move.w  me_x,d0                ;
                move.w  me_y,d1                ;
                cmp.w   nasty_x(a2),d0         ;
                beq     mn_16                  ;He's on the same x line
                bgt     mn_3                   ;
                                               ;
                tst.w   valid_left             ;
                beq     mn_3                   ;
                addq.w  #1,d5                  ;
                bset    #1,d6                  ;Travel left to get him
                bra     mn_4                   ;
mn_3:                                          ;
                tst.w   valid_right            ;
                beq     mn_4                   ;
                addq.w  #1,d5                  ;
                bset    #0,d6                  ;Travel right to get him
mn_4:                                          ;
                cmp.w   nasty_y(a2),d1         ;
                beq     mn_15                  ;He's on the same y line
                bgt     mn_5                   ;
                tst.w   valid_up               ;
                beq     mn_6                   ;
                addq.w  #1,d5                  ;
                bset    #3,d6                  ;Travel up to get him
                bra     mn_6                   ;
mn_5:                                          ;
                tst.w   valid_down             ;
                beq     mn_6                   ;
                addq.w  #1,d5                  ;
                bset    #2,d6                  ;Travel down to get him
mn_6:                                          ;

*****************************************************
* d5 equals the number of valid choices there are.  *
* d6 equals the valid directions that we can choose.*
* 0000                                              *
* |||+-right                                        *
* ||+--left                                         *
* |+---down                                         *
* +----up                                           *
*****************************************************
                tst.w   d5             ;Is there no valid way to get
                beq     mn_7           ;you ?
                                       ;
                btst    #3,d6          ;
                bne     mn_9           ;Go up to get him
                                       ;
                btst    #2,d6          ;
                bne     mn_10          ;Go down to get him
                                       ;
                btst    #1,d6          ;
                bne     mn_11          ;Go left to get him
*********************************************
* Force the nasty to go right to get to you *
*********************************************
mn_12:
                move.w  #2,nasty_dir(a2)
                bra     mn_next

*********************************************
* Force the nasty to go left to get to you  *
*********************************************
mn_11:
                move.w  #3,nasty_dir(a2)
                bra     mn_next

*********************************************
* Force the nasty to go down to get to you  *
*********************************************
mn_10:
                move.w  #1,nasty_dir(a2)
                bra     mn_next

*********************************************
* Force the nasty to go up to get to you    *
*********************************************
mn_9:
                clr.w   nasty_dir(a2)
                bra     mn_next

****************************************************************
* There is no way to head directly for you, so the nastie must *
* pick the first available direction.                          *
****************************************************************
mn_7:
                lea     RANDOM_DATA,a6
                move.w  RANDOM_OFFSET,d0
                addq.w  #1,d0
                and.w   #$0fff,d0
                move.w  d0,RANDOM_OFFSET
                move.b  (a6,d0.w),d0
                and.w   #$00ff,d0

                cmp.b   #$40,d0
                ble     .l_2
                cmp.b   #$80,d0
                ble     .l_1
                cmp.b   #$c0,d0
                ble     .l_3

                tst.w   valid_up
                bne     mn_9
.l_1:
                tst.w   valid_down
                bne     mn_10
.l_2:
                tst.w   valid_left
                bne     mn_11
.l_3:
                tst.w   valid_right
                bne     mn_12

                bra     mn_7
**********************************
* Ooops, a dead end (or a bug !) *
**********************************
mn_14:
                tst.w   nasty_dir(a2)
                beq     mn_10
                cmp.w   #1,nasty_dir(a2)
                beq     mn_9
                cmp.w   #2,nasty_dir(a2)
                beq     mn_11
                bra     mn_12

****************************************************
* He's on the same line as you, so just go for him *
****************************************************
mn_15:
                cmp.w   nasty_x(a2),d0
                blt     mn_17
                tst.w   valid_right
                beq     mn_7
                bra     mn_12
mn_17:
                tst.w   valid_left
                beq     mn_7
                bra     mn_11

mn_16:
                cmp.w   nasty_y(a2),d1
                bgt     mn_18
                tst.w   valid_up
                beq     mn_7
                bra     mn_9
mn_18:
                tst.w   valid_down
                beq     mn_7
                bra     mn_10

mn_next:
                move.w  nasty_dir(a2),d0
                tst.w   d0
                beq     mn_move_up
                cmp.w   #1,d0
                beq     mn_move_down
                cmp.w   #2,d0
                beq     mn_move_right
mn_move_left:
                subq.w  #1,nasty_x(a2)
                bra     mn_next_2
mn_move_up:
                subq.w  #1,nasty_y(a2)
                bra     mn_next_2
mn_move_down:
                addq.w  #1,nasty_y(a2)
                bra     mn_next_2
mn_move_right:
                addq.w  #1,nasty_x(a2)
mn_next_2:
                addq.l  #nasty_rs_length,a2
                dbf     d7,mn_1
                rts

* PROPORTIONAL FONT DISPLAYER
* uses sprites instead of screen memory
* Entry:
* a0 - Text address
*      1st word - X value -1 = Centred
*      2nd word - Y value
*      3rd word - text
*                 if 0 then terminate
*                 if -1 then re-read x,y at next even address

pro_print:      lea     pro_inf,a1
                lea     pro_obs,a3
.srch:          cmp.l   #-1,(a3)
                beq.s   .ok
                addq.l  #4,a3
                bra.s   .srch
.ok:            move.l  #pro_data,d0
                and.b   #$f8,d0
                lsl.l   #8,d0
                move.l  d0,a2
.re_read:       move.l  a0,d0
                addq.l  #1,d0
                and.b   #$fe,d0
                move.l  d0,a0
                movem.w (a0)+,d0-d1
                tst.w   d0
                bpl.s   .more
                bsr     centre_text
.more:          moveq   #0,d2
                moveq   #0,d3
                move.b  (a0)+,d2
                beq     .fin
                bmi     .re_read
                sub.w   #' ',d2
                lsl.w   #2,d2
                move.l  (a1,d2.w),d2
                move.b  d2,d3
                lsr.l   #8,d2
                tst.b   d2
                beq     .next_char
                movem.l d0-d3/a0-a3,-(a7)
                moveq   #0,d0
                move.b  d2,d0
                lsr.w   #5,d0
                addq.w  #4,d0
                moveq   #-1,d1
                bsr     add_object
                movem.l (a7)+,d0-d3/a0-a3
                move.l  d7,(a3)+
                add.l   #shadow_base,d7
                move.l  d7,a4
                clr.b   d2
                add.l   a2,d2
                or.l    d2,(a4)
                move.w  d1,d2
                lsl.w   #3,d2
                or.w    d2,6(a4)
                or.w    d0,14(a4)
.next_char:     add.w   d3,d0
                bra     .more
.fin:           move.l  #-1,(a3)
                rts
centre_text:    movem.l d1/a0,-(a7)
                moveq   #0,d0
.more:          move.b  (a0)+,d1
                beq.s   .fin
                bmi.s   .fin
                ext.w   d1
                sub.w   #' ',d1
                lsl.w   #2,d1
                move.l  (a1,d1.w),d1
                ext.w   d1
                add.w   d1,d0
                bra.s   .more
.fin:           lsr.w   #1,d0
                move.w  #350,d1
                sub.w   d0,d1
                move.w  d1,d0
                movem.l (a7)+,d1/a0
                rts
del_fnt_objs:   bsr     wait_int
                lea     pro_obs,a0
del_fnt_obs2:   move.l  (a0),-(a7)
                move.l  #-1,(a0)+
.more:          move.l  (a0)+,d0
                bmi.s   .ex
                move.l  a0,-(a7)
                bsr     del_object
                move.l  (a7)+,a0
                bra.s   .more
.ex:            move.l  (a7)+,d0
                bsr     del_object
                rts
                .qphrase
point_table:
        .dc.w       150,70,100,0
        .dc.w       160,70,100,0
        .dc.w       170,70,100,0
        .dc.w       160,70,90,1
        .dc.w       160,70,110,1

        .dc.w       150,80,100,0
        .dc.w       160,80,100,0
        .dc.w       170,80,100,0
        .dc.w       160,80,90,1
        .dc.w       160,80,110,1

        .dc.w       150,90,100,0
        .dc.w       160,90,100,0
        .dc.w       170,90,100,0
        .dc.w       160,90,90,1
        .dc.w       160,90,110,1

        .dc.w       150,100,100,0
        .dc.w       160,100,100,0
        .dc.w       170,100,100,0
        .dc.w       160,100,90,1
        .dc.w       160,100,110,1

        .dc.w       148,110,100,0
        .dc.w       160,110,100,0
        .dc.w       172,110,100,0
        .dc.w       160,110,88,1
        .dc.w       160,110,112,1

        .dc.w       145,120,100,0
        .dc.w       160,120,100,0
        .dc.w       175,120,100,0
        .dc.w       160,120,85,1
        .dc.w       160,120,115,1

        .dc.w       137,130,100,0
        .dc.w       160,130,100,0
        .dc.w       183,130,100,0
        .dc.w       160,130,77,1
        .dc.w       160,130,123,1

        .dc.w       127,140,100,0
        .dc.w       160,140,100,0
        .dc.w       193,140,100,0
        .dc.w       160,140,67,1
        .dc.w       160,140,133,1
        .dc.w       $ffff,0,0,0

point_table2:
        .dc.w       105,130,100,3
        .dc.w       110,120,100,3
        .dc.w       115,110,100,3

        .dc.w       120,100,100,3
        .dc.w       130,100,100,3
        .dc.w       140,100,100,3
        .dc.w       150,100,100,3
        .dc.w       160,100,100,3
        .dc.w       170,100,100,3

        .dc.w       170,090,090,3
        .dc.w       170,090,110,3
        .dc.w       170,080,080,3
        .dc.w       170,080,120,3
        .dc.w       170,070,075,3
        .dc.w       170,070,125,3
        .dc.w       170,060,075,3
        .dc.w       170,060,125,3
        .dc.w       170,050,075,3
        .dc.w       170,050,125,3
        .dc.w       170,050,075,3
        .dc.w       170,050,125,3

        .dc.w       120,090,090,3
        .dc.w       120,090,110,3
        .dc.w       120,080,080,3
        .dc.w       120,080,120,3
        .dc.w       120,070,075,3
        .dc.w       120,070,125,3
        .dc.w       120,060,075,3
        .dc.w       120,060,125,3
        .dc.w       120,050,075,3
        .dc.w       120,050,125,3
        .dc.w       120,050,075,3

        .dc.w       175,110,100,3
        .dc.w       175,120,110,3
        .dc.w       175,120,090,3
        .dc.w       180,120,100,3
        .dc.w       185,118,105,2
        .dc.w       185,118,095,2
        .dc.w       190,116,100,3
        .dc.w       200,112,100,3
        .dc.w       $ffff,0,0,0

point_table3:
        .dc.w       167,060,109,1
        .dc.w       174,060,100,1
        .dc.w       167,060,089,1
        .dc.w       155,060,085,1
        .dc.w       155,060,114,1
        .dc.w       143,060,089,1
        .dc.w       136,060,100,1
        .dc.w       143,060,109,1
        .dc.w       155,070,100,0

        .dc.w       167,070,109,1
        .dc.w       174,070,100,1
        .dc.w       167,070,089,1
        .dc.w       155,070,085,1
        .dc.w       155,070,114,1
        .dc.w       143,070,089,1
        .dc.w       136,070,100,1
        .dc.w       143,070,109,1

        .dc.w       167,080,109,1
        .dc.w       174,080,100,1
        .dc.w       167,080,089,1
        .dc.w       155,080,085,1
        .dc.w       155,080,114,1
        .dc.w       143,080,089,1
        .dc.w       136,080,100,1
        .dc.w       143,080,109,1

        .dc.w       148,090,088,1
        .dc.w       142,090,100,1
        .dc.w       155,090,110,1
        .dc.w       168,090,100,1
        .dc.w       162,090,088,1

        .dc.w       155,100,100,1

        .dc.w       155,110,100,1

        .dc.w       155,120,100,1

        .dc.w       155,130,100,1

        .dc.w       148,130,088,1
        .dc.w       142,130,100,1
        .dc.w       155,130,110,1
        .dc.w       168,130,100,1
        .dc.w       162,130,088,1
        .dc.w       162,130,088,1

        .dc.w       $ffff,0,0,0

point_tabl:
        .dc.w       160,100,100,1
        .dc.w       160,110,100,8
        .dc.w       160,118,100,9
        .dc.w       160,124,100,10
        .dc.w       160,128,100,11
        .dc.w       160,130,100,12
        .dc.w       160,132,100,13

        .dc.w       $ffff,0,0,1
        .phrase
ptable:              .dc.l       point_table
bnum:                .dc.w       0
save_return_addr:
                     .dc.l       0
next_node:           .dc.l       0
blank_node:          .dcb.l      512,0

matrix11:            .dc.w       0
matrix12:            .dc.w       0
matrix13:            .dc.w       0
matrix21:            .dc.w       0
matrix22:            .dc.w       0
matrix23:            .dc.w       0
matrix31:            .dc.w       0
matrix32:            .dc.w       0
matrix33:            .dc.w       0

xoffs:               .dc.w       0
yoffs:               .dc.w       0
zoffs:               .dc.w       0

offx:                .dc.w       0
offy:                .dc.w       0
offz:                .dc.w       0

sinx:                .dc.w       0
siny:                .dc.w       0
sinz:                .dc.w       0

cosx:                .dc.w       0
cosy:                .dc.w       0
cosz:                .dc.w       0

transx:              .dc.w       0
transy:              .dc.w       0
transz:              .dc.w       0

rotx:                .dc.w       0
roty:                .dc.w       0
rotz:                .dc.w       0

centx:               .dc.w       0
centy:               .dc.w       0
centz:               .dc.w       150

rotx11:              .dc.w       16384
rotx12:              .dc.w       0
rotx13:              .dc.w       0
rotx21:              .dc.w       0
rotx22:              .dc.w       16384
rotx23:              .dc.w       0
rotx31:              .dc.w       0
rotx32:              .dc.w       0
rotx33:              .dc.w       16384

sintab: .dc.w       0,286,572,857,1143,1428,1713,1997,2280
        .dc.w       2563,2845,3126,3406,3686,3964,4240,4516
        .dc.w       4790,5063,5334,5604,5872,6138,6402,6664
        .dc.w       6924,7182,7438,7692,7943,8192,8438,8682
        .dc.w       8923,9162,9397,9630,9860,10087,10311,10531
        .dc.w       10749,10963,11174,11381,11585,11786,11982,12176
        .dc.w       12365,12551,12733,12911,13085,13255,13421,13583
        .dc.w       13741,13894,14044,14189,14330,14466,14598,14726
        .dc.w       14849,14962,15082,15191,15296,15396,15491,15582
        .dc.w       15668,15749,15826,15897,15964,16026,16083,16135
        .dc.w       16182,16225,16262,16294,16322,16344,16362,16374
        .dc.w       16382,16384

        .dc.w       16382,16374,16362,16344,16322,16294,16262,16225
        .dc.w       16182
        .dc.w       16135,16083,16026,15964,15897,15826,15749,15668
        .dc.w       15582,15491,15396,15296,15191,15082,14962,14849
        .dc.w       14726,14598,14466,14330,14189,14044,13894,13741
        .dc.w       13853,13421,13255,13085,12911,12733,12551,12365
        .dc.w       12176,11982,11786,11585,11381,11174,10963,10749
        .dc.w       10531,10311,10087,9860,9630,9397,9162,8923
        .dc.w       8282,8438,8192,7943,7692,7438,7182,6924
        .dc.w       6664,6402,6138,5872,5604,5334,5063,4790
        .dc.w       4516,4240,3964,3686,3406,3126,2845,2563
        .dc.w       2280,1997,1713,1428,1143,857,572,286,0


        .dc.w       -286,-572,-857,-1143,-1428,-1713,-1997,-2280
        .dc.w       -2563,-2845,-3126,-3406,-3686,-3964,-4240,-4516
        .dc.w       -4790,-5063,-5334,-5604,-5872,-6138,-6402,-6664
        .dc.w       -6924,-7182,-7438,-7692,-7943,-8192,-8438,-8682
        .dc.w       -8923,-9162,-9397,-9630,-9860,-10087,-10311,-10531
        .dc.w       -10749,-10963,-11174,-11381,-11585,-11786,-11982
        .dc.w       -12176
        .dc.w       -12365,-12551,-12733,-12911,-13085,-13255,-13421
        .dc.w       -13583
        .dc.w       -13741,-13894,-14044,-14189,-14330,-14466,-14598
        .dc.w       -14726
        .dc.w       -14849,-14962,-15082,-15191,-15296,-15396,-15491
        .dc.w       -15582
        .dc.w       -15668,-15749,-15826,-15897,-15964,-16026,-16083
        .dc.w       -16135
        .dc.w       -16182,-16225,-16262,-16294,-16322,-16344,-16362
        .dc.w       -16374,-16382,-16384


        .dc.w       -16382,-16374,-16362,-16344,-16322,-16294,-16262
        .dc.w       -16225,-16182
        .dc.w       -16135,-16083,-16026,-15964,-15897,-15826,-15749
        .dc.w       -15668
        .dc.w       -15582,-15491,-15396,-15296,-15191,-15082,-14962
        .dc.w       -14849
        .dc.w       -14726,-14598,-14466,-14330,-14189,-14044,-13894
        .dc.w       -13741
        .dc.w       -13853,-13421,-13255,-13085,-12911,-12733,-12551
        .dc.w       -12365
        .dc.w       -12176,-11982,-11786,-11585,-11381,-11174,-10963
        .dc.w       -10749
        .dc.w       -10531,-10311,-10087,-9860,-9630,-9397,-9162,-8923
        .dc.w       -8282,-8438,-8192,-7943,-7692,-7438,-7182,-6924
        .dc.w       -6664,-6402,-6138,-5872,-5604,-5334,-5063,-4790
        .dc.w       -4516,-4240,-3964,-3686,-3406,-3126,-2845,-2563
        .dc.w       -2280,-1997,-1713,-1428,-1143,-857,-572,-286,0
        .even


PIXEL_TABLE_LEFT:
                .dc.w       %1111111111111111
                .dc.w       %0111111111111111
                .dc.w       %0011111111111111
                .dc.w       %0001111111111111
                .dc.w       %0000111111111111
                .dc.w       %0000011111111111
                .dc.w       %0000001111111111
                .dc.w       %0000000111111111
                .dc.w       %0000000011111111
                .dc.w       %0000000001111111
                .dc.w       %0000000000111111
                .dc.w       %0000000000011111
                .dc.w       %0000000000001111
                .dc.w       %0000000000000111
                .dc.w       %0000000000000011
                .dc.w       %0000000000000001
PIXEL_TABLE_RIGHT:
                .dc.w       %1000000000000000
                .dc.w       %1100000000000000
                .dc.w       %1110000000000000
                .dc.w       %1111000000000000
                .dc.w       %1111100000000000
                .dc.w       %1111110000000000
                .dc.w       %1111111000000000
                .dc.w       %1111111100000000
                .dc.w       %1111111110000000
                .dc.w       %1111111111000000
                .dc.w       %1111111111100000
                .dc.w       %1111111111110000
                .dc.w       %1111111111111000
                .dc.w       %1111111111111100
                .dc.w       %1111111111111110
                .dc.w       %1111111111111111
                .phrase
space:               .dcb.b      32128,0


control_txt:
                .dc.b   4*3,8*20,8
                .dc.b   'You can play with the balls',-1
                .dc.b   4*3,8*21,8
                .dc.b   '1,2,3:object 4,5,6: X+,Y+,Z+',-1
                .dc.b   4*3,8*22,8
                .dc.b   '7,8,9: -X,-Y,-Z  0: reset to 0',-1
                .dc.b   4*3,8*23,8
                .dc.b   'Joy Up: +speed  Joy Down: -Speed',0

instr_txt:
                .dc.b   4*2,8*9,8
                .dc.b   'The objective of the game is to fill',-1
                .dc.b   4*2,8*10,8
                .dc.b   'all the boxes by outlining them while',-1
                .dc.b   4*2,8*11,8
                .dc.b   'avoiding the nasty blue monsters.',-1
                .dc.b   4*2,8*12,8
                .dc.b   'You have one defense against the ',-1
                .dc.b   4*2,8*13,8
                .dc.b   'monsters: You can cut holes in the',-1
                .dc.b   4*2,8*14,8
                .dc.b   'lines, but you can only have 3 holes',-1
                .dc.b   4*2,8*15,8
                .dc.b   'open at the same time. The holes will',-1
                .dc.b   4*2,8*16,8
                .dc.b   'repair after a short time.',-1
                .dc.b   4*2,8*17,8
                .dc.b   'Press the B button to cut holes.',-1
                .dc.b   4*2,8*18,8
                .dc.b   'The trick to playing this game is to',-1
                .dc.b   4*2,8*19,8
                .dc.b   'hold the joypad in the direction',-1
                .dc.b   4*2,8*20,8
                .dc.b   'that you want to go next and holding',-1
                .dc.b   4*2,8*21,8
                .dc.b   'it in a upper left diagonal will ',-1
                .dc.b   4*2,8*22,8
                .dc.b   'make you go up/left/up/left etc',0
                .even

credit_txt:
                .dc.b   4*6,8*9,2
                .dc.b   'Game Coding:',-1
                .dc.b   4*20,8*9,8
                .dc.b   'Gordon Gibson,',-1
                .dc.b   4*20,8*10,8
                .dc.b   'David Crawford &',-1
                .dc.b   4*20,8*11,8
                .dc.b   'Dave Cowan',-1
                .dc.b   4*4,8*12,2
                .dc.b   'Voice Effects:',-1
                .dc.b   4*20,8*12,8
                .dc.b   'DeeGee',-1
                .dc.b   4*5,8*13,2
                .dc.b   'Level Design:',-1
                .dc.b   4*20,8*13,8
                .dc.b   'Gordon Gibson',-1
                .dc.b   4*3,8*14,2
                .dc.b   'Bitmap Artwork:',-1
                .dc.b   4*20,8*14,8
                .dc.b   'Alasdair MacMillan',-1
                .dc.b   4*20,8*15,8
                .dc.b   '& Mike Watson',-1
                .dc.b   4*4,8*16,2
                .dc.b   'Music Modules:',-1
                .dc.b   4*20,8*16,8
                .dc.b   'Kenny Tait',-1
                .dc.b   4*6,8*17,2
                .dc.b   'Playtesting:',-1
                .dc.b   4*20,8*17,8
                .dc.b   'Gordon Gibson,',-1
                .dc.b   4*20,8*18,8
                .dc.b   'Carl Forhan',-1
                .dc.b   4*20,8*19,8
                .dc.b   '& Stone',0
                .even

startup_txt:    .dc.b   4*8,8*10,8
                .dc.b   '[   ] Enter Level Password',-1
                .dc.b   4*8,8*12,8
                .dc.b   '[   ] Play Painter',-1
                .dc.b   4*8,8*14,8
                .dc.b   '[   ] Credits',-1
                .dc.b   4*8,8*16,8
                .dc.b   '[   ] Instructions',-1
                .dc.b   4*8,8*20,8
                .dc.b   'High Score: 00000000       ',-1
                .dc.b   4*8,8*22,8
                .dc.b   'Last Score: 00000000       ',-1
                .dc.b   4*10,8*10,2,'A',-1
                .dc.b   4*10,8*12,2,'B',-1
                .dc.b   4*10,8*14,2,'C',-1
                .dc.b   4*10,8*16,2,'1',-1
                .dc.b   4*20,8*20,8
menu_score1:    .dc.b   '00123456',-1
                .dc.b   4*20,8*22,8
menu_score2:    .dc.b   '00123456'
cheat:          .dc.b   0
                .dc.b   4*11,8*19,2
                .dc.b   'CHEAT MODE ACTIVE!',0
                .even

startup_txt2:   .dc.b   4*8,8*10,8
                .dc.b   '[   ] Enter Level Password',-1
                .dc.b   4*8,8*12,8
                .dc.b   '[   ] Play Painter',-1
                .dc.b   4*8,8*14,8
                .dc.b   '[   ] Credits',-1
                .dc.b   4*8,8*16,8
                .dc.b   '[   ] Instructions',-1
                .dc.b   4*8,8*18,8
                .dc.b   'High Score: 00000000       ',-1
                .dc.b   4*8,8*22,8
                .dc.b   'Last Score: 00000000       ',-1
                .dc.b   4*10,8*10,2,'A',-1
                .dc.b   4*10,8*12,2,'B',-1
                .dc.b   4*10,8*14,2,'C',-1
                .dc.b   4*10,8*16,2,'1',-1
                .dc.b   4*20,8*20,8
menu_score3:    .dc.b   '00123456',-1
                .dc.b   4*20,8*22,8
menu_score4:    .dc.b   '00123456'
                .dc.b   0
                .dc.b   4*11,8*19,2
                .dc.b   'CHEAT MODE ACTIVE!',0
                .even

passwd_txt:     .dc.b   2*13,8*10,8
                .dc.b   'P A S S W O R D   E N T R Y',-1
                .dc.b   2*6,8*12,8
                .dc.b   'Press [UP/DOWN] to change letter',-1
                .dc.b   2*6,8*13,8
                .dc.b   'Press [LEFT]    to delete letter',-1
                .dc.b   2*6,8*14,8
                .dc.b   'Press [RIGHT]   to enter letter ',-1
                .dc.b   2*6,8*15,8
                .dc.b   'Press [A/B/C]   to check password',-1
                .dc.b   2*20,8*12,2
                .dc.b   'UP/DOWN',-1
                .dc.b   2*20,8*13,2
                .dc.b   'LEFT',-1
                .dc.b   2*20,8*14,2
                .dc.b   'RIGHT',-1
                .dc.b   2*20,8*15,2
                .dc.b   'A/B/C',0
passwd_txt2:    .dc.b   2*32,8*18,2
passwd_asc:     .dc.b   0,0,0,0,0,0,0,0,0
                .even
passwd_pos:     .dc.w   0
                .even

font:           .incbin  "font.jag"

gamelogo:       .incbin  "glogo.jp3"
gamelogo_end:
cpyrt:          .incbin  "tcr.jp3"

intro_pal:      .dc.w   $0000,$8008,$e230,$0620
                .dc.w   $0518,$0410,$0308,$0208
                .dc.w   $0730,$6000,$0100,$a010
                .dc.w   $a018,$c120,$e128,$0628

scr_obj:        .dc.l   0
              .phrase
time_loop_value:
                .dc.w   0
final_levels:   .dc.w   0
box_height:     .dc.w   0
me_x:           .dc.w   0
me_y:           .dc.w   0
me_dir:         .dc.w   3
line_colour:    .dc.w   0
no_of_boxes_left:
                .dc.w   0
no_of_boxes:    .dc.w   0
valid_up:       .dc.w   0
valid_down:     .dc.w   0
valid_left:     .dc.w   0
valid_right:    .dc.w   0
flip_address:   .dc.l   0
flip_data:      .dc.w   0
erase_address:  .dc.l   0
erase_d0:       .dc.l   0
clear_address:  .dc.l   0
clear_data:     .dc.l   0
set_address:    .dc.l   0
set_data:       .dc.l   0
erase_pixel_flag:
                .dc.w   0
move_occur:     .dc.w   0
add_score:      .dc.l   0
old_score:      .dc.l   0
extra_score:    .dc.l   $100            *       10,000

hole_table:     .dcb.l  3*4*3,0
                .dc.l   "EGB!"

sprite_address:         .dc.l   0
box_stack:              .dcb.w  4*(4+1),0
player_dead:            .dc.w   0
totally_dead_dude:      .dc.w   0
no_of_nasties:          .dcb.w  1,0
check_x:                .dc.w   0
check_y:                .dc.w   0
xline_flag:             .dc.w   0
yline_flag:             .dc.w   0
skip_level_flag:        .dc.w   0

*       Empty space is where the levels are loaded and stored
*       the first (150*32)+4 bytes are the level offsets :

*       0       l       Offset to Level coords
*       4       l       Offset to Level Nasty data
*       8       w       Timer for Level
*       10      w       No of Nasties for level
*       12-32   w       Name (ASCII)

*       The level coords are simply a list of coordinates, held
*       (x1,y1),(x2,y2),(x3,y3),(x4,y4) and then a score for that box :
*
*       0       w       x1                              \
*       2       w       y1                              |
*       4       w       x2                              |
*       6       w       y2                              |
*       8       w       x1                              |
*       10      w       y1                              |       *
*       12      w       x2                              |
*       14      w       y2                              |
*       16      l       BCD Longword Score for Box      |
*       20      w       Hide Box (0=No, 1=Yes)          |
*       22                                              /

*       The Nasty data is a type number (0-3), and a start coordinate
*       for the beast.

*       The previous two words, however, are the start coordinates for
*       the player.

*
*       -12     8       password for level
*       -4      w       player X
*       -2      w       player Y
*Nasty_data
*       0       w       type                    \
*       2       w       x                       |       *
*       4       w       y                       /


LEVEL_TEXT:
                .dc.w   -1,0
                .dc.b   "Level  "
LEVEL_NUMBER_TEXT:
                .dc.b   "000",-1
                .even
TIME_TEXT:      .dc.w   -1,120*2
                .dc.b   "Time : "
LEVEL_TIME_TEXT:
                .dc.b   "XXX sec's",-1
                .even
LEVEL_NAME_TEXT:
                .dc.w   -1,40*2
LEVEL_NAME:
                .dcb.b  20,0
                .dc.b   0
                .even
PASSWORD_TEXT:  .dc.w   -1,80*2
                .dc.b   "Password : "
PASSWORD_TEXT_STRING:
                .dcb.b  8,0
                .dc.b   0
                .even

GAME_COMPLETE_TEXT_DEMO:
                .dc.b   4*12,8*10,2
                .dc.b   "Congratulations ",-1
                .dc.b   4*2,8*12,3
                .dc.b   "You have completed the demo version",-1
                .dc.b   4*2,8*13,3
                .dc.b   "of Painter. To get the full version",-1
                .dc.b   4*2,8*14,3
                .dc.b   "please visit our web site:-",-1
                .dc.b   4*4,8*15,3
                .dc.b   " http:\\\\sinister.kewlplace.com",-1
                .dc.b   4*2,8*17,3
                .dc.b   "As a small bonus try holding OPTION",-1
                .dc.b   4*2,8*18,3
                .dc.b   "& STAR on the main intro screen!",-1
                .dc.b   4*2,8*20,3
                .dc.b   "Regards, Gordon Gibson.",0

                .phrase

GAME_COMPLETE_TEXT:
                .dc.b   4*2,8*7,2
                .dc.b   "Final Score with lives left bonus:",-1
                .dc.b   4*2,8*8,8
                .dc.b   "              "
life_bonus:     .dc.b   "00000000",-1
                .dc.b   4*12,8*10,2
                .dc.b   "Congratulations ",-1
                .dc.b   4*2,8*12,3
                .dc.b   "You have completed all of the levels",-1
                .dc.b   4*2,8*13,3
                .dc.b   "of Painter. To play more levels    ",-1
                .dc.b   4*2,8*14,3
                .dc.b   "use ORIGINAL or DEMO as a password.",-1
                .dc.b   4*2,8*15,3
                .dc.b   "use STANDARD for default levels.",-1
                .dc.b   4*4,8*17,3
                .dc.b   "You can also visit our website at",-1
                .dc.b   4*4,8*18,3
                .dc.b   " http:\\\\www.kewlplace.com",-1
                .dc.b   4*2,8*19,3
                .dc.b   "As a small bonus try holding OPTION",-1
                .dc.b   4*2,8*20,3
                .dc.b   "& STAR on the main intro screen!",0

                .phrase

GAME_OVER_TEXT:
                .dc.w   -1,76*2
                .dc.b   "Game Over",0

                .phrase

GAME_ABORTED_TEXT:
                .dc.w   -1,76*2
                .dc.b   "Game Aborted",0
                .phrase

WOW_TEXT:       .dc.w   -1,0
                .dc.b   "Level Complete !",-1
WOW2_TEXT:      .dc.w   -1,70*2
                .dc.b   "--Time Bonus--",0
                .phrase

TIME_BONUS_PRINT:       .dc.w   -1,130*2
                        .dc.b   "100 x "
BONUS_TIME:             .dc.b   "000 = "
BONUS_SCORE:            .dc.b   "00000",0
                .phrase

blank_text:     .dcb.b  40,0

LINE_TEXT:      .dc.b   1*4,0,7
                .dc.b   'SCORE:',-1
                .dc.b   19*4,0,7
                .dc.b   'TIME:',-1
                .dc.b   31*4,0,7
                .dc.b   'LIVES:',-1
                .dc.b   37*4,0,6
lives_asc:      .dc.b   '00 ',0

                .even
dummy_space:    .dc.b   ' '
score_text:     .dc.b   7*4,0,6
score_asc:      .dc.b   '????????',0

time_text:      .dc.b   24*4,0,6
time_asc:       .dc.b   '???',0
                .even

TIMEOUT_TEXT:   .dc.w   -1,76*2
                .dc.b   "Timeout !",0
                .even

NO_MORE_PLAY_TEXT:
                .dc.b   0,0
                .dc.b   'W E L L   D O N E  ! !',0
                .qphrase
sorted_points:          .dcb.l  500,0
                .qphrase
sorted_points2:         .dcb.l  500,0
                .qphrase
xspace:                 .dcb.l  100,0
level_space:
level_file_data:        .dcb.b   64000,0
                        .phrase
level_file_data_1:      .incbin  "newlvl.lvl"
                        .dcb.b   256,0
                        .phrase
level_file_data_2:      .incbin  "demo.lvl"
                        .dcb.b   256,0
                        .phrase
level_file_data_3:      .incbin  "levels.lvl"
                        .dcb.b   256,0
                        .phrase
level_file:             .dc.w   0
                        .phrase
RANDOM_DATA:            .incbin  "random.dat"

game_pal:               .dc.w   $0000,$0720,$4210,$6318
colour_4:               .dc.w   $0000,$a528,$e738,$8420
                        .dc.w   $e000,$0038,$e000,$2108
colour_12:              .dc.w   $e000,$0710,$0508,$0300
pro_pal:                .dc.w   $0000,$2108,$4210,$6318
                        .dc.w   $8420,$a528,$c630,$e738
                        .dc.w   $0000,$0000,$0000,$0000
                        .dc.w   $0000,$0000,$0000,$0000
white_pal:              .dc.w   $ffff,$ffff,$ffff,$ffff
                        .dc.w   $ffff,$ffff,$ffff,$ffff
                        .dc.w   $ffff,$ffff,$ffff,$ffff
                        .dc.w   $ffff,$ffff,$ffff,$ffff
                        .phrase
player_obj:             .dc.l   0
nasty_obj1:             .dc.l   0
nasty_obj2:             .dc.l   0
nasty_obj3:             .dc.l   0
nasty_obj4:             .dc.l   0
nasty_obj5:             .dc.l   0
ball_obj:               .dcb.l  100,0

RANDOM_OFFSET:          .dc.w   0


nasty_x                 .equ     0      ;offset to x
nasty_y                 .equ     2      ;offset to y
nasty_dir               .equ     4      ;offset to dir
nasty_type              .equ     6      ;offset to type
nasty_rs_length         .equ     8      ;start of next nasty 8 bytes on
        .even

box_table:      .dcb.w  40,0            ;no_of_pixels
                .dcb.w  40,0            ;box_x1
                .dcb.w  40,0            ;box_x2
                .dcb.w  40,0            ;box_y1
                .dcb.w  40,0            ;box_y2
                .dcb.l  40,0            ;point_value
                .dcb.w  40,0            ;spare

no_of_levels:           .dc.w   0
nasty_space:            .dcb.b  40,0

level_name_text:        .dcb.b  32,0

                        .even
GO_MED_REZ:             .dc.w   0
GAME_ABORTED:           .dc.w   0
timeout_message_flag:   .dc.w   0

BOLLOX:                 .dc.w   0
TIME_FRAME:             .dc.w   0

TIME_COUNT_1:           .dc.w   0
TIME_COUNT_2:           .dc.w   0
TIME_COUNT_3:           .dc.w   0
TIME_LEFT:              .dc.w   0
HA_HA_NO_MORE_PLAY_TODAY:       .dc.w   0
START_LEVEL:            .dc.w   0
LEVEL_NUMBER:           .dc.w   0
NO_OF_LIVES:            .dc.w   0
LIVES:                  .dc.w   0
SCORE:                  .dc.l   0
K_U:                    .dc.b   0
K_D:                    .dc.b   0
K_L:                    .dc.b   0
K_R:                    .dc.b   0
K_F:                    .dc.b   0
K_1:                    .dc.b   0
K_3:                    .dc.b   0
K_A:                    .dc.b   0
K_FR:                   .dc.b   0
move_priority:          .dc.b   0
hidden:                 .dc.b   0
                .even
bonus:                  .dc.w   0
                .qphrase
raw_obj_data:

dummy_obj:
.o_xpos:        .dc.w   0
.o_ypos:        .dc.w   0
.o_data:        .dc.l   blank_bit
.o_height:      .dc.w   1
.o_dwidth:      .dc.w   1
.o_iwidth:      .dc.w   1
.o_flags:       .dc.b   %1100                  ;rel/trans/add/ref
.o_firstpix:    .dc.b   0
.o_type:        .dc.b   0
.o_depth:       .dc.b   0
.o_pitch:       .dc.b   1
.o_index:       .dc.b   0
.o_desc:        .dc.w   0
.scaling_info1: .dc.b   $00
.scaling_info3: .dc.b   $00
.scaling_info4: .dc.b   $00
.scaling_info5: .dc.b   $00
.o_rom:         .dc.l   0
.o_mode:        .dc.l   0

fireworks_scr:  .dc.w   scr_x
                .dc.w   scr_y
                .dc.l   0
                .dc.w   200
                .dc.w   20
                .dc.w   20
                .dc.b   %1000
                .dc.b   0
                .dc.b   0
                .dc.b   2
                .dc.b   1
                .dc.b   0
                .dc.w   0
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.l   0
                .dc.l   0

_320x200x16:    .dc.w   scr_x
                .dc.w   scr_y
                .dc.l   screen_32k
                .dc.w   200
                .dc.w   20
                .dc.w   20
                .dc.b   %1100
                .dc.b   0
                .dc.b   0
                .dc.b   2
                .dc.b   1
                .dc.b   0
                .dc.w   0
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.l   0
                .dc.l   0

painter_spr:    .dc.w   124
                .dc.w   196
                .dc.l   sprites
                .dc.w   16
                .dc.w   1
                .dc.w   1
                .dc.b   %1100
                .dc.b   0
                .dc.b   0
                .dc.b   2
                .dc.b   1
                .dc.b   0
                .dc.w   0
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.l   0
                .dc.l   0

fnt_16x32:      .dc.w   0
                .dc.w   0
                .dc.l   0
                .dc.w   32
                .dc.w   1
                .dc.w   1
                .dc.b   %1100
                .dc.b   0
                .dc.b   0
                .dc.b   2
                .dc.b   1
                .dc.b   0
                .dc.w   0
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.l   0
                .dc.l   0

fnt_32x32:      .dc.w   0
                .dc.w   0
                .dc.l   0
                .dc.w   32
                .dc.w   2
                .dc.w   2
                .dc.b   %1100
                .dc.b   0
                .dc.b   0
                .dc.b   2
                .dc.b   1
                .dc.b   0
                .dc.w   0
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.l   0
                .dc.l   0

ball_obj1:      .dc.w   18
                .dc.w   100
                .dc.l   blank_bit
                .dc.w   16
                .dc.w   2
                .dc.w   2
                .dc.b   %1100
                .dc.b   0
                .dc.b   0
                .dc.b   3
                .dc.b   1
                .dc.b   0
                .dc.w   0
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.b   $00
                .dc.l   0
                .dc.l   0

paused_obj:     .dc.w    40                      ; o_xpos
                .dc.w    196                     ; o_ypos
                .dc.l    pause_spr               ; o_data
                .dc.w    23                      ; o_height
                .dc.w    55                      ; o_dwidth
                .dc.w    55                      ; o_iwidth
                .dc.b    %0000                   ; o_flags rel/trans/add/ref
                .dc.b    0                       ; o_firstpix
                .dc.b    0                       ; o_type
                .dc.b    4                       ; o_depth
                .dc.b    1                       ; o_pitch
                .dc.b    0                       ; o_index
                .dc.w    0                       ; o_desc
                .dc.b    $00                     ; scaling info
                .dc.b    $00                     ; scaling info
                .dc.b    $00                     ; scaling info
                .dc.b    $00                     ; scaling info
                .dc.l    0                       ; o_rom
                .dc.l    0                       ; o_mode


                .phrase
branch_object:  .dc.w   0
                .dc.w   0
                .dc.w   0
                .dc.w   $0002
                .dc.l   0
                .dc.l   4
                .dc.l   0
                .dc.l   0
                .dc.l   0
                .dc.l   0

curr_objects:   .dc.w   0               ;no. of objects in use
next_object:    .dc.l   0               ;pointer to next available addr
obj_list_end:   .dc.l   0
obj_ptr_table:  .dcb.l  num_objects,0   ;stack of pointers
end_ptrs:       .dc.l   -1              ;terminated by -1
stop_object:    .dc.l   0
                .phrase
pre_def_objects:
                .dcb.l  100*8,0
pro_inf:        .incbin  "pro_fnt.inf"
                .phrase
sprites:        .incbin  "sprites.js2"
                .phrase
pro_data:       .incbin  "pro_fnt.d8a"
                .phrase
ball_data:      .incbin  "balls.jag"
                .phrase
ball_pal:       .incbin  "balls.pal"
                .phrase
pause_spr:      .incbin  "oppause.jag"
                .phrase
music_spr:      .incbin  "opmusic.jag"
                .phrase
fx_spr:         .incbin  "opfx.jag"
                .phrase
arrow_spr:      .incbin  "oparrow.jag"
                .phrase
red_spr:        .incbin  "opmulti.jag"
                .phrase
fx_vol_spr:     .incbin  "opmulti.jag"
                .phrase
mus_vol_spr:    .incbin  "opmulti.jag"
                .phrase
samp1:          .incbin "egb.pcm"
samp1_end:      .phrase
samp2:          .incbin "secs20.raw"
samp2_end:      .phrase
samp3:          .incbin "secs10.raw"
samp3_end:      .phrase
samp4:          .incbin "secs5.raw"
samp4_end:      .phrase
samp5:          .incbin "secs0.raw"
samp5_end:      .phrase
samp6:          .incbin "aborted.raw"
samp6_end:      .phrase
samp7:          .incbin "cut.raw"
samp7_end:      .phrase
samp8:          .incbin "ding.raw"
samp8_end:      .phrase
samp9:          .incbin "paused.raw"
samp9_end:      .phrase
samp10:         .incbin "stan.raw"
samp10_end:     .phrase
samp11:         .incbin "demo.raw"
samp11_end:     .phrase
samp12:         .incbin "orig.raw"
samp12_end:     .phrase
samp13:         .incbin "beep.raw"
samp13_end:     .phrase
samp14:         .incbin "munch.raw"
samp14_end:     .phrase
samp15:         .incbin "cheer.raw"
samp15_end:     .phrase
samp16:         .incbin "move1.raw"
samp16_end:     .phrase
samp17:         .incbin "move2.raw"
samp17_end:     .phrase
samp18:         .incbin "move3.raw"
samp18_end:     .phrase
samp19:         .incbin "move4.raw"
samp19_end:     .phrase
samp20:         .dc.l   0
samp20_end:     .phrase
samp_table:     .dc.l    samp1_end-samp1, samp1
                .dc.l    samp2_end-samp2, samp2
                .dc.l    samp3_end-samp3, samp3
                .dc.l    samp4_end-samp4, samp4
                .dc.l    samp5_end-samp5, samp5
                .dc.l    samp6_end-samp6, samp6
                .dc.l    samp7_end-samp7, samp7
                .dc.l    samp8_end-samp8, samp8
                .dc.l    samp9_end-samp9, samp9
                .dc.l    samp10_end-samp10, samp10
                .dc.l    samp11_end-samp11, samp11
                .dc.l    samp12_end-samp12, samp12
                .dc.l    samp13_end-samp13, samp13
                .dc.l    samp14_end-samp14, samp14
                .dc.l    samp15_end-samp15, samp15
                .dc.l    samp16_end-samp16, samp16
                .dc.l    samp17_end-samp17, samp17
                .dc.l    samp18_end-samp18, samp18
                .dc.l    samp19_end-samp19, samp19
mod_table:      .dc.l    modi
                .dc.l    modb
                .dc.l    modc
                .dc.l    modd
                .dc.l    mode
                .dc.l    modf
                .dc.l    modg
                .dc.l    modh
                .dc.l    moda
                .dc.l    modj
                .dc.l    modk
                .dc.l    modl
moda:           .incbin  "a.mod"
                .phrase
modb:           .incbin  "b.mod"
                .phrase
modc:           .incbin  "c.mod"
                .phrase
modd:           .incbin  "d.mod"
                .phrase
mode:           .incbin  "e.mod"
                .phrase
modf:           .incbin  "f.mod"
                .phrase
modg:           .incbin  "g.mod"
                .phrase
modh:           .incbin  "h.mod"
                .phrase
modi:           .incbin  "i.mod"
                .phrase
modj:           .incbin  "j.mod"
                .phrase
modk:           .incbin  "k.mod"
                .phrase
modl:           .incbin  "l.mod"
                .qphrase
screen_32k:     .dcb.b  32000,0
screen_32ka:    .dcb.b  64000,0
screen_32kb:    .dcb.b  64000,0
                .qphrase
blank_bit:      .dcb.b  1000,0
                .phrase
obj_stop:       .dc.l    0,4

scrbase:        .dc.l    0
scrswap:        .dc.l    0
scrtemp:        .dc.l    0
a_vdb:          .dc.w    0
a_vde:          .dc.w    0
intcount:       .dc.w    0
uplock:         .dc.w    0
dnlock:         .dc.w    0
zero_press:     .dc.w    1
zero_lock:      .dc.w    0
starttext:      .dc.w    0
psamp:          .dc.w    0
pausesamp:      .dc.w    0
beepsamp:       .dc.w    0
movesamp:       .dc.w    0
mod_loop:       .dc.w    0
curr_mod:       .dc.l    0
map_obj_adds:   .dc.l    0
no_copy:        .dc.w    0
joyedge:        .dcb.l   2,0
joycur:         .dcb.l   2,0
vbl:            .dc.l    0
control:        .dc.b    0
blank_pal:      .dcb.w   16,0
pro_obs:        .dcb.l   40*40,0
